#include "ppdfviewer.h"

#include <qapplication.h>
#include <qvbox.h>
#include <qpainter.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qpaintdevicemetrics.h>
#include <qprinter.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qcursor.h>
#include <qdragobject.h>
#include <qfiledialog.h>
#include <qtimer.h>
#include <qobjectlist.h>
#include <qprogressdialog.h>
#include <qfontmetrics.h>
//#include <qtextbrowser.h>

#if defined (Q_OS_WIN32)
	#include <qaxwidget.h>
//	#include <windows.h>
//	#include <tlhelp32.h>
#endif

#include "pdfinclude/GlobalParams.h"
#include "pdfinclude/PDFDoc.h"
#include "pdfinclude/SplashBitmap.h"
#include "pdfinclude/SplashOutputDev.h"
//#include "pdfinclude/PSOutputDev.h"
#include "pdfinclude/TextOutputDev.h"

// Grafiken fr ToolBar-Buttons in XPM-Format
/* columns rows colors chars-per-pixel */

/* XPM */
const char* PPdfViewer::pdfFitHV[]={
"20 22 4 1",
"# c #5c5555",
"a c #b2abab",
"b c #ffffff",
". c None",
"###################.",
"#aaaaaaaaaaaaaaaaa#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbbb###bbbbbba#.",
"#abbbbb#####bbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#abb#bbbbbbbbb#bba#.",
"#ab##bbbbbbbbb##ba#.",
"#a######bbb######a#.",
"#ab##bbbbbbbbb##ba#.",
"#abb#bbbbbbbbb#bba#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#abbbbb#####bbbbba#.",
"#abbbbbb###bbbbbba#.",
"#abbbbbbb#bbbbbbba#.",
"#aaaaaaaaaaaaaaaaa#.",
"###################.",
"...................."};

/* XPM */
const char* PPdfViewer::pdfFitH[]={
"20 22 4 1",
". c #5c5555",
"a c #b2abab",
"b c #ffffff",
"# c None",
"...................#",
".aaaaaaaaaaaaaaaaa.#",
".abbbbbbbbbbbbbbba.#",
".abbbbbbbbbbbbbbba.#",
".abb.bbbbbbbbb.bba.#",
".ab..bbbbbbbbb..ba.#",
".a......bbb......a.#",
".ab..bbbbbbbbb..ba.#",
".abb.bbbbbbbbb.bba.#",
".abbbbbbbbbbbbbbba.#",
".abbbbbbbbbbbbbbba.#",
".abbbbbbbbbbbbbbba.#",
".abb.bbbbbbbbb.bba.#",
".ab..bbbbbbbbb..ba.#",
".a......bbb......a.#",
".ab..bbbbbbbbb..ba.#",
".abb.bbbbbbbbb.bba.#",
".abbbbbbbbbbbbbbba.#",
".abbbbbbbbbbbbbbba.#",
".aaaaaaaaaaaaaaaaa.#",
"...................#",
"####################"};

/* XPM */
const char* PPdfViewer::pdfPageUp[]={
"20 22 7 1",
"# c #5c5555",
"b c #9c9595",
"a c #b2abab",
"c c #c5bebe",
"d c #e9e2e2",
"w c #ffffff",
". c None",
"###################.",
"#aaaaaaaaaaaaaaaaa#.",
"#awwwwwww##wwwwwwa#.",
"#awwwwww#b#wwwwwwa#.",
"#awwwww#bc#wwwwwwa#.",
"#awwww#bdc#wwwwwwa#.",
"#awww#bddc#wwwwwwa#.",
"#aww#bdddc#########.",
"#aw#bddddccccccccc#.",
"#a#bdddddddddddddd#.",
"##bddddddddddddddd#.",
"#a#bdddddddddddddd#.",
"#aw#bddddccccccccc#.",
"#aww#bdddc#########.",
"#awww#bddc#wwwwwwa#.",
"#awwww#bdc#wwwwwwa#.",
"#awwwww#bc#wwwwwwa#.",
"#awwwwww#b#wwwwwwa#.",
"#awwwwwww##wwwwwwa#.",
"#aaaaaaaaaaaaaaaaa#.",
"###################.",
"...................."};

/* XPM */
const char* PPdfViewer::pdfPageDown[]={
"20 22 7 1",
"# c #5c5555",
"b c #9c9595",
"a c #b2abab",
"c c #c5bebe",
"d c #e9e2e2",
"w c #ffffff",
". c None",
"###################.",
"#aaaaaaaaaaaaaaaaa#.",
"#awwwwww##wwwwwwwa#.",
"#awwwwww#b#wwwwwwa#.",
"#awwwwww#cb#wwwwwa#.",
"#awwwwww#cdb#wwwwa#.",
"#awwwwww#cddb#wwwa#.",
"#########cdddb#wwa#.",
"#cccccccccddddb#wa#.",
"#ddddddddddddddb#a#.",
"#dddddddddddddddb##.",
"#ddddddddddddddb#a#.",
"#cccccccccddddb#wa#.",
"#########cdddb#wwa#.",
"#awwwwww#cddb#wwwa#.",
"#awwwwww#cdb#wwwwa#.",
"#awwwwww#cb#wwwwwa#.",
"#awwwwww#b#wwwwwwa#.",
"#awwwwww##wwwwwwwa#.",
"#aaaaaaaaaaaaaaaaa#.",
"###################.",
"...................."};

/* XPM */
const char* PPdfViewer::pdfZoomPlus[]={
"14 14 4 1",
". c #005500",
"b c #adbea5",
"a c #e8ffdd",
"# c #f4f47a",
"..............",
".############.",
".#aaaaaaaaaa#.",
".#aaaabbaaaa#.",
".#aaaa..aaaa#.",
".#aaaa..aaaa#.",
".#ab......ba#.",
".#ab......ba#.",
".#aaaa..aaaa#.",
".#aaaa..aaaa#.",
".#aaaabbaaaa#.",
".#aaaaaaaaaa#.",
".############.",
".............."};

/* XPM */
const char* PPdfViewer::pdfZoomMinus[]={
"14 14 4 1",
". c #005500",
"b c #adbea5",
"a c #e8ffdd",
"# c #f4f47a",
"..............",
".############.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".#ab......ba#.",
".#ab......ba#.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".#aaaaaaaaaa#.",
".############.",
".............."};

/* XPM */
const char* PPdfViewer::pdfPrint[]={
"22 22 42 1",
"c c #302929",
"o c #3f3838",
"v c #403939",
"n c #504949",
"e c #5f5858",
"b c #605959",
"u c #635c5c",
"a c #6f6868",
"# c #736c6c",
"f c #787171",
"N c #837c7c",
"s c #8c8585",
"p c #8f8888",
"m c #938c8c",
"M c #9b9494",
"K c #9c9595",
"t c #9f9898",
"L c #a49d9d",
"J c #aca5a5",
"I c #b4adad",
"G c #bbb4b4",
"H c #bcb5b5",
"j c #c3bcbc",
"D c #cbc4c4",
"F c #ccc5c5",
"h c #cfc8c8",
"i c #d0c9c9",
"y c #d4cdcd",
"C c #dcd5d5",
"l c #dfd8d8",
"q c #e3dcdc",
"E c #e4dddd",
"w c #ece5e5",
"r c #f0e9e9",
"k c #f3ecec",
"B c #f4eded",
"A c #fbf4f4",
"g c #fff8f8",
"d c #fff9f9",
"z c #fffcfc",
"x c #fffdfd",
". c None",
"......................",
"......................",
"....#aaaaaabca........",
"....a......dcce.......",
"....a.ffff.gchce......",
"....a......gcijce.....",
"....a.ffff.gccccc.....",
"....a.......kkklc.....",
"...ma.ffffff...kcno...",
"..pqa..........rcsc...",
"..mtmmmmmmmmmmmmmpuv..",
".pw..............xyc..",
".#zzzzzzzzzzzzzzzAqc..",
".aAB..wwwwwwwwwwwCDcc.",
".akw..EEEEEEEEEEEyjcc.",
".awECCCCCCCCCCCCCFGcc.",
".eCFHHHHHHHHHHHHHIJcc.",
".cKLMMMMMMMMMMMMMMNcc.",
"..cccccccccccccccccc..",
"...ccccccccccccccccc..",
"......................",
"......................"};

/* XPM */
const char* PPdfViewer::pdfFileSaveAs[] = {
"22 22 3 1",
"# c #5c5555",
"a c #b2abab",
". c None",
"......................",
".####################.",
".#aa#............#..#.",
".#aa#............#..#.",
".#aa#............####.",
".#aa#............#aa#.",
".#aa#............#aa#.",
".#aa#............#aa#.",
".#aa#............#aa#.",
".#aa#............#aa#.",
".#aa#............#aa#.",
".#aaa############aaa#.",
".#aaaaaaaaaaaaaaaaaa#.",
".#aaaaaaaaaaaaaaaaaa#.",
".#aaa#############aa#.",
".#aaa#########...#aa#.",
".#aaa#########...#aa#.",
".#aaa#########...#aa#.",
".#aaa#########...#aa#.",
".#aaa#########...#aa#.",
"..##################..",
"......................"};

/* XPM */
const char* PPdfViewer::pdfFileOpen[] = {
"22 22 4 1",
"# c #635555",
"b c #a59a9a",
"a c #efe2e2",
". c None",
"......................",
"......................",
"............####....#.",
"...........#....##.##.",
"..................###.",
".................####.",
".####...........#####.",
"#aaaa##########.......",
"#aaaaaaaaaaaaa#.......",
"#aaaaaaaaaaaaa#.......",
"#aaaaaaaaaaaaa#.......",
"#aaaaaa###############",
"#aaaaa##bbbbbbbbbbbb##",
"#aaaa##bbbbbbbbbbbb##.",
"#aaa##bbbbbbbbbbbb##..",
"#aa##bbbbbbbbbbbb##...",
"#a##bbbbbbbbbbbb##....",
"###bbbbbbbbbbbb##.....",
"##bbbbbbbbbbbb##......",
"###############.......",
"......................",
"......................"};

/* XPM */
const char* PPdfViewer::pdfSymbol[]={
"22 22 32 1",
". c #000000",
"D c #080808",
"n c #161616",
"z c #330000",
"C c #660000",
"y c #800000",
"x c #990000",
"m c #993333",
"o c #999999",
"p c #9999cc",
"A c #99cccc",
"v c #b2b2b2",
"q c #c0c0c0",
"i c #cc0000",
"d c #cc3333",
"j c #cc3366",
"e c #cc6666",
"B c #cc6699",
"k c #cc9999",
"l c #cc99cc",
"r c #cccccc",
"s c #ccffff",
"t c #dddddd",
"a c none",//#e0dfe3",
"u c #eaeaea",
"b c #ff3333",
"g c #ff3366",
"c c #ff6666",
"f c #ff6699",
"h c #ff9999",
"w c #ffcccc",
"# c none",
"....##....##....######",
".###.#.##.##.#########",
".###.#.###.#...#######",
"....##.###.#.#########",
".#####.##.##.#########",
".#####....##.#########",
"######aaaaaaabbaaaaaaa",
"######aaaaaaacdaaaaaaa",
"######efcffcgfdehcciaa",
"######jkkkkkkkclkkkmna",
"######jopqrss#bh##kmna",
"######jopqtt#udd##kmna",
"######jovrtt#cqwb#kina",
"######dkqrtuud#wddcdbx",
"######dkrrtscehktukdiy",
"######dqrtrcd#####kixz",
"######drsredr#uuuukmia",
"######dtsedvsrttttkmna",
"######iuutrtrqrrrrkmna",
"######i###ssttrAAAkmna",
"######iBBBBBBBBBBBBmna",
"######mCCCCCCCCCCCCCDa"};

/* XPM */
const char* PPdfViewer::pdfGrayScale[]={
"20 22 10 1",
"z c #5c5555",
"x c #ffffff",
"e c #b1b1b1",
"y c #b7b7b7",
"# c #bebebe",
"a c #c4c4c4",
"c c #cbcbcb",
"b c #d1d1d1",
"d c #d8d8d8",
". c None",
"zzzzzzzzzzzzzzzzzzz.",
"zxxxxxxxxxxxxxxxxxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxbba#yacdae#abdcxz.",
"zxxxxxxxxxxxxxxxxxz.",
"zzzzzzzzzzzzzzzzzzz.",
"...................."};

/* XPM */
const char* PPdfViewer::pdfColor[]={
"20 22 23 1",
"z c #5c5555",
"x c #ffffff",
"r c #8a8aff",
"q c #8a9eff",
"p c #8ad8ff",
"k c #8aff8a",
"l c #8affb1",
"m c #8affc5",
"n c #8affec",
"o c #8affff",
"# c #b18aff",
"a c #c58aff",
"j c #c5ff8a",
"s c #d88aff",
"b c #ec8aff",
"e c #ff8ab1",
"d c #ff8ac5",
"c c #ff8aec",
"f c #ff9e8a",
"g c #ffc58a",
"h c #ffd88a",
"i c #ffff8a",
". c None",
"zzzzzzzzzzzzzzzzzzz.",
"zxxxxxxxxxxxxxxxxxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxabceghijklmopqrxz.",
"zxxxxxxxxxxxxxxxxxz.",
"zzzzzzzzzzzzzzzzzzz.",
"...................."};

/* XPM */
const char* PPdfViewer::pdfText[]={
"22 22 13 1",
"c c #c5bebe",
"a c #e8e2e2",
"e c #000000",
"j c #4d4d4d",
". c #5c5555",
"h c #686868",
"g c #7c7c7c",
"d c #9e9a9a",
"k c #a7a7a7",
"# c #c5bebe",
"f c #d0d0d0",
"b c #e8e2e2",
"i c #f0f0f0",
"......................",
".####################.",
".#aaaaaaaaaaaaaabbbb#.",
".#bbbbbbbbbbbbbbbbbb#.",
".#...........aabbbbb#.",
".#...........aabbbbbc.",
".#dddd..dddddaabbbbbc.",
".#abbb..dabbbabbbbbbc.",
".#bbbb..dabbbbbbbbbbc.",
".#bbbb..dabbbbbbbbbbc.",
".#bbbb..dabbbbbbbbbbc.",
".#bbbb..dabbbbbbbbbbc.",
".#bbbb..dabbbbbbbbebc.",
".#bbbb..dafefbgfgeeec.",
".#bbbb..dahbhbfefbeac.",
".#bbbb..daeeebieibeac.",
".#bbbb..dajbbbfefbeac.",
".#bbbb..dafekbgfgbgec.",
".#bbbb..dabbbbbbbbbb#.",
".#bbbbbbbabbbbbbbbbb#.",
".####################.",
"......................"};

/* XPM */
const char* PPdfViewer::pdfHtml[]={
"22 22 5 1",
". c #5c5555",
"# c #c5bebe",
"b c #e8e2e2",
"c c #5c5555",
"a c #e8e2e2",
"......................",
".####################.",
".#aaaaaaaabaacaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccaccaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccaccaaaaa#.",
".#bc.aaaaaccaaaaaaaa#.",
".#bc.aaaaaccacccccca#.",
".#bcccccccccacaacaac#.",
".#bcccccccccacaacaac#.",
".#bc.aaaaaccacaacaac#.",
".#bc.aaaaaccacaacaac#.",
".#bc.aaaaaccaaaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#bc.aaaaaccacaaaaaa#.",
".#baaaaaaaaaaccaaaaa#.",
".####################.",
"......................"};

/* XPM */
const char* PPdfViewer::pdfJpg[]={
"22 22 11 1",
"c c #857b7b",
"i c #c5bebe",
"a c #e8e2e2",
". c #5c5555",
"d c #9e9a9a",
"h c #aba7a7",
"# c #c5bebe",
"g c #c6c4c4",
"f c #cecccc",
"e c #d5d3d3",
"b c #e8e2e2",
"......................",
".####################.",
".#abbbbbbbbbbbbbbbbb#.",
".#abbbbbbbbbbb.b..bb#.",
".#aabbbbbb..cb..bb.b#.",
".#babbbbbb..cb.bbb.b#.",
".#aabbbbbb..cb.bbb.b#.",
".#abbbbbbb..cb..bb.b#.",
".#abbbbbbb..cb.b..bb#.",
".#abbbbbbb..cb.bbbbb#.",
".#abbbbbbb..cb.bbbbb#.",
".#abbbbbbb..cbb..b.b#.",
".#abbbbbbb..cb.bb..b#.",
".#a...bbbb..cb.bbb.b#.",
".#ad..ebbe..db.bbb.b#.",
".#af........gb.bb..b#.",
".#abh......hbbb..b.b#.",
".#abbgd..hebbbbbbb.b#.",
".#abbbbbbbbbbb....bb#.",
".#abbbbbbbbbbbbbbbbb#.",
".############iiiii###.",
"......................"};

const char* PPdfViewer::pdfCaption[]={
"16 12 2 1",
". c #7c5555",
"# c None",
"................",
"################",
"#....###########",
"#.###.####.###.#",
"#.###.####.##.##",
"#.###.##...#...#",
"#....##.##.##.##",
"#.#####.##.##.##",
"#.#####.##.##.##",
"#.######...##.##",
"################",
"................"};


PButton::PButton(QString str, QWidget *parent) : QPushButton(str, parent)
{
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	cp.setColor( QPalette::Inactive, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	setPalette(cp);
	setFocusPolicy(QWidget::NoFocus);
	setCursor(QCursor::QCursor(Qt::PointingHandCursor));
	QFont fo(font());
	fo.setBold(true);
	setFont(fo);
	if(QColor::numBitPlanes()>8) setPaletteBackgroundColor(QColor::QColor(210,205,205));
}

PButton::PButton(const QIconSet &icon, QString str, QWidget *parent) : QPushButton(icon, str, parent)
{
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	cp.setColor( QPalette::Inactive, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	setPalette(cp);
	setFocusPolicy(QWidget::NoFocus);
	setCursor(QCursor::QCursor(Qt::PointingHandCursor));
	QFont fo(font());
	fo.setBold(true);
	setFont(fo);
	if(QColor::numBitPlanes()>8) setPaletteBackgroundColor(QColor::QColor(210,205,205));
}

void PButton::focusInEvent(QFocusEvent *ev)
{
	QWidget::focusInEvent(ev);
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(255,0,0));
	setPalette(cp);
}

void PButton::focusOutEvent(QFocusEvent * ev)
{
	QWidget::focusOutEvent(ev);
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	setPalette(cp);
}

void PButton::enterEvent(QEvent *ev)
{
	QWidget::enterEvent(ev);
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(255,0,0));
	setPalette(cp);
}

void PButton::leaveEvent(QEvent *ev)
{
	QWidget::leaveEvent(ev);
	QPalette cp = palette();
	cp.setColor( QPalette::Active, QColorGroup::ButtonText, QColor::QColor(80,0,0));
	setPalette(cp);
}

PToolButtonRep::PToolButtonRep(QWidget *parent) : QToolButton(parent)
{
	timer = new QTimer(this);
	startInt = 200;
	setToggleButton(true);
	setToggleType(QButton::SingleShot);
	setOn(false);
	setCursor(QCursor::QCursor(Qt::PointingHandCursor));
	stopMe = false;
}

void PToolButtonRep::setInterval(int millisec)
{
	startInt = millisec;
}

void PToolButtonRep::repeateSignal()
{
	if(stopMe) { return; }
	emit downHolded();
}

void PToolButtonRep::mousePressEvent(QMouseEvent *ev)
{
	stopMe = false;
	QToolButton::mousePressEvent(ev);
	if(ev->button()==Qt::LeftButton) {
		emit downHolded();
		QObject::connect( timer, SIGNAL(timeout()), this, SLOT(repeateSignal()) );
		timer->start(startInt, false);
		setOn(true);
	}
}

void PToolButtonRep::mouseReleaseEvent(QMouseEvent *ev)
{
	stopMe = true;
	QObject::disconnect( timer, SIGNAL(timeout()), this, SLOT(repeateSignal()) );
	timer->stop(); timer = new QTimer(this);
	QToolButton::mouseReleaseEvent(ev);
	setOn(false);
}

void PToolButtonRep::setEnabled(bool b)
{
	QWidget::setEnabled(b);
	if(!b) {
		stopRepeate();
	}
}

void PToolButtonRep::stopRepeate()
{
	stopMe = true;
	QObject::disconnect( timer, SIGNAL(timeout()), this, SLOT(repeateSignal()) );
	timer->stop();
	timer = new QTimer(this);
	setOn(false);
	setDown(false);
}

PPdfView::PPdfView(QWidget* parent, const char* name, WFlags f) : QScrollView(parent, name, f)
{
	globalParams = new GlobalParams("xpdfrc.cfg"); // "xpdfrc.cfg" muss nicht existieren
	globalParams->setErrQuiet(true);
	globalParams->setupBaseFonts(NULL);
//	bool b;
//	b = globalParams->setEnableT1lib("yes");
//	b = globalParams->setEnableFreeType("yes");
//	b = globalParams->setAntialias("yes");
//	b = globalParams->setVectorAntialias("yes");

	//setBackgroundMode(Qt::NoBackground);
	//setBackgroundMode(Qt::FixedPixmap);

	bWithAdobe = false;

	bColor = false;

	dataFromFile = 0;
	dataLen = 0;

	viewPixmap = new QPixmap;
	//viewPixmap->setOptimization(QPixmap::MemoryOptim); //QPixmap::BestOptim);

	hbTop = 0;

	imageList.setAutoDelete(true);

	deltaV = 16;
	deltaH = 6;

	fileNamePdf = "";
	resolutionStd = 72;
	resolutionCur = 72;

	pageCurrent = 0;
	pageCount = 0;
	pageHeight = 0;

	ownerPW = "";
	userPW = "";

	#if defined (Q_OS_WIN32)
		reader = 0;
	#endif

	//verticalScrollBar()->setTracking(true);
	//verticalScrollBar()->setLineStep(QApplication::wheelScrollLines());
	QObject::connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(updateView()));
	QObject::connect(horizontalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(updateView()));

	setAcceptDrops(true);

	QObject::connect(this, SIGNAL(droppedFileName(const QString&)), this, SLOT(load(const QString&)));

//	setFont(QFont("Helvetica", 8, QFont::Normal));
//	#if defined (Q_WS_X11)
//		setFont(QFont("Helvetica", 10, QFont::Normal));
//	#endif
//	#if defined (Q_WS_MACX)
//		setFont(QFont("Helvetica", 10, QFont::Normal));
//	#endif
}

PPdfView::~PPdfView()
{
	if(dataFromFile) delete dataFromFile;
	if(viewPixmap) delete viewPixmap;
	if(hbTop) delete hbTop;
	imageList.clear();
	#if defined (Q_OS_WIN32)
		if(reader) delete reader;
	#endif
}

void PPdfView::setLeftMargin(int leftMargin)
{
	deltaH = leftMargin;
}

void PPdfView::setPageDistance(int pageDistance)
{
	deltaV = pageDistance;
}

void PPdfView::updateView()
{
	if(imageList.count()>0) {
		int h = (*imageList.at(0)).height()+deltaV;
		double hS = viewport()->height()/12.0*5.0;
		pageCurrent = (contentsY()+(int)hS) / h + 1;
	}
	viewport()->setUpdatesEnabled(true);
	viewport()->repaint();
	viewport()->setUpdatesEnabled(false);
	if(imageList.count()>0) {
		emit pageChanged(" Seite: "+QString::number(pageCurrent)+" / "+QString::number(pageCount)+" ");
	}
}

void PPdfView::enableAdobeReader()
{
	bWithAdobe = true;
}

void PPdfView::disableAdobeReader()
{
	bWithAdobe = false;
}

void PPdfView::init()
{
	#if defined (Q_OS_WIN32)

		if(bWithAdobe && !reader) {

			reader = new QAxWidget(this, "Acrobat Reader");

			reader->show();
			bool b = reader->setControl("{CA8A9780-280D-11CF-A24D-444553540000}"); // Acrobat Reader 4, 5, 6, 7, 8, 9
			if(!b) {
				b = reader->setControl("{05BFD3F1-6319-4F30-B752-C7A22889BCC4}"); // ??? Acrobat Reader 7, 8, ...
			}
			if(!b) {
				delete reader;
				reader = 0;
			}
			if(b) {
				//QTextBrowser* ted = new QTextBrowser(0);
				//ted->setText(reader->generateDocumentation());
				//ted->show();
				reader->dynamicCall( "setShowToolbar(bool)", true );
				reader->resize(size());
				return;
			}
		}
		if(reader) {
			delete reader;
			reader = 0;
		}

	#endif
}

void PPdfView::load(const QString& fileName)
{
	load(fileName, FitHV);

	updateView();
}

int PPdfView::load(QString fileName, int layout, double resolution, int mode, QString ownerPassword, QString userPassword)
{
	init();

	#if defined (Q_OS_WIN32)
		if(bWithAdobe && reader) {
			bool b = reader->dynamicCall("LoadFile(const QString&)", fileName).toBool();
			fileNamePdf = fileName;
			ownerPW = ownerPassword;
			userPW  = userPassword;
			if(layout==FitHV) { setView("Fit");	 }
			if(layout==FitH)  {	setView("FitH"); }
			return b;
		}
	#endif

	char* data;
	QFile f; f.setName(fileName);
	QFileInfo fi; fi.setFile(f);
	if(f.open(IO_ReadOnly)) {
		QDataStream dStr(&f);
		data = new char[fi.size()];
		dStr.readRawBytes(data,fi.size());
		f.close();
		if(dataFromFile!=0) delete dataFromFile;
		dataLen = fi.size();
		fileNamePdf = fileName;
		ownerPW = ownerPassword;
		userPW  = userPassword;
	} else {
		return 0;
	}

	return loadFromData(data, fi.size(), layout, resolution, mode, ownerPassword, userPassword);
}

void PPdfView::setData(QString fileName, int layout, QString ownerPassword, QString userPassword)
{
	char* data;
	QFile f; f.setName(fileName);
	QFileInfo fi; fi.setFile(f);
	if(f.open(IO_ReadOnly)) {
		QDataStream dStr(&f);
		data = new char[fi.size()];
		dStr.readRawBytes(data,fi.size());
		f.close();
		if(dataFromFile!=0) delete dataFromFile;
		dataLen = fi.size();
		fileNamePdf = fileName;

		setData(data, dataLen, layout, ownerPassword, userPassword);
	}
}

void PPdfView::setData(char* data, uint len, int layout, QString ownerPassword, QString userPassword)
{
	dataFromFile = data;
	dataLen = len;
	viewLayout = layout;
	ownerPW = ownerPassword;
	userPW  = userPassword;
}

void PPdfView::loadData()
{
	loadFromData(dataFromFile, dataLen, viewLayout);
	updateView();
}

int PPdfView::loadFromData(char* data, uint len, int layout, double resolution, int mode, QString ownerPassword, QString userPassword)
{
	if(bWithAdobe) { init(); }

	dataFromFile = data;

	viewLayout = layout;

	if(resolution<1) { resolution = 1; }
	if(len<1) { return 0; }

	GString* ownerPass, * userPass;
	SplashColor paperColor;
	SplashOutputDev* splashOut;
	int pg;

	if(ownerPassword!="") {
		ownerPass = new GString(ownerPassword.ascii());
	} else {
		ownerPass = NULL;
	}
	if(userPassword!="") {
		userPass = new GString(userPassword.ascii());
	} else {
		userPass = NULL;
	}
	
	Object obj;
	MemStream* mStr;
	PDFDoc* doc;

	obj.initNull();
	mStr = new MemStream(data, 0, len, &obj);
	doc = new PDFDoc(mStr, ownerPass, userPass);

	if(userPass)  { delete userPass; }
	if(ownerPass) { delete ownerPass; }

	if(!doc->isOk()) {

		delete doc;
		Object::memCheck(stderr);
		gMemReport(stderr);

		return 0;
	}

	resolutionCur = resolution;
	imageList.clear();

/*
  SplashOutputDev(SplashColorMode colorModeA, int bitmapRowPadA,
		  GBool reverseVideoA, SplashColorPtr paperColorA,
		  GBool bitmapTopDownA = gTrue,
		  GBool allowAntialiasA = gTrue);
*/

	SplashColorMode modeC;

	if(bColor || mode>23) {
		modeC = splashModeRGB8;
		paperColor[0] = paperColor[1] = paperColor[2] = 0xff;
		splashOut = new SplashOutputDev(splashModeRGB8, 1, gFalse, paperColor);
	} else {
		if(mode<8) {
			modeC = splashModeMono1;
			paperColor[0] = 0xff;
			splashOut = new SplashOutputDev(splashModeMono1, 1, gFalse, paperColor);
		}
		if(mode>7 && mode<24) {
			modeC = splashModeMono8;
			paperColor[0] = 0xff;
			splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);
		}
	}

	splashOut->startDoc(doc->getXRef());

	int h = deltaV;
	int w = 200, wMax = 0;
	SplashBitmap* bp = 0;
	int firstPage = 1;
	int lastPage = doc->getNumPages();

	pageCurrent = 1;
	pageCount = lastPage;

	for(pg=firstPage;pg<=lastPage;pg++) {

		if(pg==firstPage) {
			if(layout!=FitNo) {

				// void displayPage(OutputDev *out, int page, double hDPI, double vDPI, int rotate, GBool useMediaBox, GBool crop,
				//					 GBool printing, GBool (*abortCheckCbk)(void *data) = NULL, void *abortCheckCbkData = NULL);
				doc->displayPage(splashOut, pg, resolution, resolution, 0, gFalse, gTrue, gFalse);

				bp = splashOut->getBitmap();

				int f = resolutionCur;
				if(layout==FitHV) {
					double wI = bp->getWidth();
					double hI = bp->getHeight();
					f  = (width()-30)  / wI * resolutionCur;
					int fV = (height()-20) / hI * resolutionCur;
					if(fV<f) { f = fV; }
				}
				if(layout==FitH) {
					double wI = bp->getWidth();
					f  = (width()-30)  / wI * resolutionCur;
				}

				resolution = f;
				resolutionCur = f;
			}
		}

		doc->displayPage(splashOut, pg, resolution, resolution, 0, gFalse, gTrue, gFalse);

		bp = splashOut->getBitmap();

		QImageIO iio;
		QImage* image;

		QByteArray a;
		QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

		SplashColorPtr row, p;
		int x, y;	
		row = bp->getDataPtr();

		if(modeC==splashModeMono1) {

			QString tx = "P4\n";
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			for(y=0;y<bp->getHeight();y++) {
				p = row;
				for(x=0;x<bp->getWidth();x+=8) {
					*stream << (*p ^ 0xff);
					p++;
				}
				row += bp->getRowSize();
			}
		}
		if(modeC==splashModeMono8) {

			QString tx = "P5\n";
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n255\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
		}
		if(modeC==splashModeRGB8) {

			QString tx = "P6\n";
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n255\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
		}

		image = new QImage(a);
		imageList.append(image);

		delete stream;
		a.resetRawData( a.data(), a.size() );

		h += image->height() + deltaV;

		w = image->width();
		if(w>wMax) { wMax = w; }

		resizeContents(wMax+deltaV, h);
		updateScrollBars();

		qApp->processEvents(200);
	}

	emit sendPageCount(lastPage);

	pageHeight = w;

	delete splashOut;
	delete doc;
	Object::memCheck(stderr);
	gMemReport(stderr);

	return h;
}

void PPdfView::gotoPrevPage()
{
	if(bWithAdobe) {
		return;
	}

	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<1) { return; }

	if(pageCurrent>1) pageCurrent--;
	gotoPage(pageCurrent);
}

void PPdfView::gotoNextPage()
{
	if(bWithAdobe) {
		return;
	}

	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<1) { return; }

	if(pageCurrent<pageCount) pageCurrent++;
	gotoPage(pageCurrent);
}

void PPdfView::gotoPage(int page)
{
	if(bWithAdobe) {
		return;
	}

	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<1) { return; }

	if(page>pageCount) page = pageCount;
	if(page<1) page = 1;
	pageCurrent = page;
	int h = (*imageList.at(0)).height() + deltaV;
	h = h * (page-1) ;

	setContentsPos(contentsX(), viewport()->y() + h);

	pageCurrent = page;

	viewport()->setUpdatesEnabled(true);
	viewport()->repaint();
	viewport()->setUpdatesEnabled(false);
	emit pageChanged(" Seite: "+QString::number(pageCurrent)+" / "+QString::number(pageCount)+" ");
}

void PPdfView::setSizeH(double horizontalW)
{
	if(bWithAdobe) {
		setView("FitH");
		return;
	}

	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<1) { return; }

	bool b = false;
	if(contentsY()<10) {
		b = true;
	}

	int yyA = contentsY() + viewport()->height()/2;
	int hA = contentsHeight();

	double wI = (*imageList.at(0)).width();
	int f = horizontalW / wI * resolutionCur;

	int pg = pageCurrent; // pageCurrent wird in loadFromData() auf 1 gesetzt
	int hB = loadFromData(dataFromFile, dataLen, false, f);
	pageCurrent = pg;

	double fa = ((double)hB)/((double)hA);
	int yyB = fa * yyA;
	if(b) {
		setContentsPos(contentsX(), 0);
	} else {
		setContentsPos(contentsX(), yyB - viewport()->height()/2);
	}

	updateView();
}

void PPdfView::setSizeHV(double horizontalW, double verticalH)
{
	if(bWithAdobe) {
		setView("Fit");
		return;
	}

	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<1) { return; }

	bool b = false;
	if(contentsY()<10) {
		b = true;
	}

	int yyA = contentsY() + viewport()->height()/2;
	int hA = contentsHeight();

	double wI = (*imageList.at(0)).width();
	double hI = (*imageList.at(0)).height();
	if(wI<1) { wI=1; }
	if(hI<1) { hI=1; }
	int f  = horizontalW / wI * resolutionCur;
	int fV = verticalH   / hI * resolutionCur;
	if(fV<f) { f = fV; }
	if(f<1) { f = 1; }

	int pg = pageCurrent; // pageCurrent wird in loadFromData() auf 1 gesetzt
	int hB = loadFromData(dataFromFile, dataLen, false, f);
	pageCurrent = pg;

	double fa = ((double)hB)/((double)hA);
	int yyB = fa * yyA;
	if(b) {
		setContentsPos(contentsX(), 0);
	} else {
		setContentsPos(contentsX(), yyB - viewport()->height()/2);
	}

	updateView();
}

void PPdfView::setSizeFitH()
{
	int w = viewport()->width();
	if(imageList.count()>0) {
		double wI = (*imageList.at(0)).width();
		double hI = (*imageList.at(0)).height();

		double hTotal = (w-2*deltaH)/wI*hI;
		int c = imageList.count();
		hTotal = c*hTotal + c*deltaV + deltaV;

		if(verticalScrollBar()->isVisible() && hTotal<viewport()->height()) {
			// der vertikale Scrollbalken wird verschwinden
			w += 20;
		}
		if(!verticalScrollBar()->isVisible() && hTotal>viewport()->height()) {
			// der vertikale Scrollbalken wird entstehen
			w -= 20;
		}
	}
	setSizeH(w-2*deltaH);
}

void PPdfView::setSizeFitHV()
{
	int w = viewport()->width();
	int h = viewport()->height();
	if(imageList.count()>0) {
		double wI = (*imageList.at(0)).width();
		double hI = (*imageList.at(0)).height();

		double factor =  (w-2*deltaH)/wI;
		double factor2 = (h-2*deltaV)/hI;
		if(factor>factor2) { factor = factor2; }
		double hTotal = factor*hI;
		int c = imageList.count();
		hTotal = c*hTotal + c*deltaV + deltaV;

		if(verticalScrollBar()->isVisible() && hTotal<viewport()->height()) {
			// der vertikale Scrollbalken wird verschwinden
			w += 20;
		}
		if(!verticalScrollBar()->isVisible() && hTotal>viewport()->height()) {
			// der vertikale Scrollbalken wird entstehen
			w -= 20;
		}
	}
	setSizeHV(w-2*deltaH, h-2*deltaV);
}

void PPdfView::drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph)
{
	if(bWithAdobe) {
		QScrollView::drawContents(p, clipx, clipy, clipw, cliph);
		return;
	}

	int count = imageList.count();
	if(count<1) {
		QScrollView::drawContents(p, clipx, clipy, clipw, cliph);
		return;
	}

	viewPixmap->fill(viewport()->backgroundColor());

	QPainter painter(viewPixmap);
	//QPainter painter(viewport());

	QImage im = *imageList.at(0);

	int iA = contentsY() / (im.height()+deltaV);
	int iE = (contentsY()+viewport()->height()-deltaV) / (im.height()+deltaV) + 1;
	if(count>iE) { count = iE; }
	int top = (contentsY()) % (im.height()+deltaV);
	top = - top + deltaV;
	int i;
	for(i=iA;i<count;i++) {

		im = *imageList.at(i);

		int left = (viewport()->width() - im.width())/2;
		if(left<deltaH) { left = deltaH; }
		if(contentsX()>0) {
			if(contentsX()<=deltaH) {
				left -= contentsX();
			} else {
				left = 0;
			}
		}

		// Seite der PDF-Datei zeichnen
		painter.drawImage(left, top, im, contentsX()-((bool)(contentsX()>deltaH))*left);

		QPen pen = painter.pen();
		QBrush brush = painter.brush();

		// Umrandung zeichnen
		if(i+1==pageCurrent) {
			painter.setPen(QColor(180,0,0));
		} else {
			painter.setPen(QColor(100,100,100));
		}
		painter.drawRect(left-1, top-1, im.width()+2 - contentsX(), im.height()+2);
		if(i+1==pageCurrent) {
			painter.setPen(QColor(205,160,160));
		} else {
			painter.setPen(QColor(180,180,180));
		}
		painter.drawRect(left-2, top-2, im.width()+4 - contentsX(), im.height()+4);

		if(deltaV>15 && imageList.count()>1) {
			// Seitenzahl zeichnen
			QFont fo("Arial", 8, QFont::Bold);
			painter.setFont(fo);
			QFontMetrics fm(fo);
			int w = fm.width("Seite "+QString::number(i+1))+5;
			if(2*w+50<im.width()) {
				if(i+1==pageCurrent) {
					painter.setPen(QColor(200,80,80));
					painter.setBrush(QColor(200,80,80));
				} else {
					painter.setPen(QColor(110,110,110));
					painter.setBrush(QColor(110,110,110));
				}
				// Seitenzahl oben links
				if(i>0) painter.drawText(left, top-4, "Seite "+QString::number(i+1));
				QPointArray a;
				a.setPoints( 3, left+w, top-deltaV+5, left+w+8, top-deltaV+5, left+w+4, top-5 );
				if(i>0) painter.drawPolygon(a);
				// Seitenzahl unten rechts
				left = im.width() + left - w;
				if(i<count-1) painter.drawText(left, top+im.height()+12, "Seite "+QString::number(i+1));
				a.setPoints( 3, left-15, top+im.height()+11, left-7, top+im.height()+11, left-11, top+im.height()+5 );
				if(i<count-1) painter.drawPolygon(a);
			} else {
				if(left>w+10) {
					if(i+1==pageCurrent) {
						painter.setPen(QColor(200,80,80));
						painter.setBrush(QColor(200,80,80));
					} else {
						painter.setPen(QColor(110,110,110));
						painter.setBrush(QColor(110,110,110));
					}
					// Seitenzahl links
					painter.drawText(left-w-10, top+im.height()/2, "Seite "+QString::number(i+1));
				}
			}
		}

		painter.setPen(pen);
		painter.setBrush(brush);

		top += im.height() + deltaV;
	}

	viewport()->setPaletteBackgroundPixmap(*viewPixmap);
	//bitBlt(viewport(), 0, 0, viewPixmap);
}

void PPdfView::contentsWheelEvent(QWheelEvent* ev)
{
	QScrollView::contentsWheelEvent(ev);
	updateView();
}

void PPdfView::viewportResizeEvent(QResizeEvent* ev)
{
	#if defined (Q_OS_WIN32)
		if(bWithAdobe && reader) {
			reader->resize(size());
			QScrollView::resizeEvent(ev);
			return;
		}
	#endif
	QScrollView::resizeEvent(ev);
	viewPixmap->resize(viewport()->size());
	updateView();
}

bool PPdfView::getAdobeReader()
{
	#if defined (Q_OS_WIN32)
		if(!reader) { return false; }
	#endif
	#if defined (Q_WS_X11)
		return false;
	#endif
	#if defined (Q_WS_MACX)
		return false;
	#endif

	return true;
}

QString PPdfView::version()
{
	QString val = "";

	#if defined (Q_OS_WIN32)
		if(!reader) { return "NO READER"; }
		val = reader->dynamicCall( "GetVersions()" ).toString();
		if(val.contains("=7.")) { return "7"; }
		if(val.contains("=8.")) { return "8"; }
		if(val.contains("=9.")) { return "9"; }
	#endif

	return val;

	//QVariant val = reader->dynamicCall( "GetVersions()" );
	//qDebug("val: "+val.toString());
	//if(val.toString()=="") {
	//	qDebug("val ist leer"); // Acrobat 4, 5, 6
	//}
	//if(val.toString().contains("=7.")) {
	//	qDebug("val zeigt version 7"); // Acrobat 7
	//}
	//if(val.toString().contains("=8.")) {
	//	qDebug("val zeigt version 8"); // Acrobat 8
	//}
	//if(val.toString().contains("=9.")) {
		//qDebug("val zeigt version 9"); // Acrobat 9
	//}
}

void PPdfView::setView(QString view)
{
	#if defined (Q_OS_WIN32)
		if(!reader) { return; }
		reader->dynamicCall("setView()", view);
	#endif

//	Fit		Fits the entire page within the window both vertically and horizontally.
//	FitH	Fits the entire width of the page within the window.
//	FitV	Fits the entire height of the page within the window.
//	FitB	Fits the bounding box within the window both vertically and horizontally.
//	FitBH	Fits the entire width of the bounding box within the window.
//	FitB	Fits the entire height of the bounding box within the window.
}

void PPdfView::setColor()
{
	if(bColor==false) {
		bColor = true;
	} else {
		bColor = false;
	}
}

void PPdfView::extractText()
{

	if(dataLen==0 || !dataFromFile) {
		return;
	}
	QString path=QDir::homeDirPath();
	#if defined (Q_OS_WIN32)
		if(path.length()<8) {
			path = "C:/Windows";
		}
	#endif
	if(path.length()>1 && (path.right(1)=="/" || path.right(1)=="\\")) {
		path = path.left(path.length()-1);
	}
	path += "/Desktop";
	path = QDir::convertSeparators(path);
	QFileInfo finf;
	finf.setFile(path);
	if(!finf.exists() || !finf.isDir()) {
		path = qApp->applicationDirPath();
	}
	QString s = QFileDialog::getSaveFileName(path, "Text-Dateien (*.txt)", this, "Text aus Datei speichern", "Text aus Datei speichern unter");
	if(!s.isNull() && s!="") {
		if(s.lower().right(4)!=".txt") { s = s + ".txt"; }
		QFile f; f.setName(s);
		if(f.exists()) {
			QMessageBox mBox("Speichern unter", "Die Datei existiert bereits. Soll diese berschrieben werden?", 
							 QMessageBox::Warning, QMessageBox::Yes, QMessageBox::Abort | QMessageBox::Default | QMessageBox::Escape, 0);
			mBox.setButtonText(QMessageBox::Yes, "Ja, speichern");
			mBox.setButtonText(QMessageBox::Abort, "Nein, abbrechen");
			int answer = mBox.exec();
			if(answer!=QMessageBox::Yes) {
				return;
			}
		}
			GString* ownerPass, * userPass;
			if(ownerPW!="") {
				ownerPass = new GString(ownerPW.ascii());
			} else {
				ownerPass = NULL;
			}
			if(userPW!="") {
				userPass = new GString(userPW.ascii());
			} else {
				userPass = NULL;
			}
			
			Object obj;
			MemStream* mStr;
			PDFDoc* doc;

			obj.initNull();
			mStr = new MemStream(dataFromFile, 0, dataLen, &obj);
			doc = new PDFDoc(mStr, ownerPass, userPass);

			if(userPass)  { delete userPass; }
			if(ownerPass) { delete ownerPass; }

			if(!doc->isOk()) {

				delete doc;
				Object::memCheck(stderr);
				gMemReport(stderr);

				return;
			}

			TextOutputDev *textOut;

			// write text file
			textOut = new TextOutputDev(GString(s).getCString(), gTrue, gTrue, gFalse);
			if (textOut->isOk()) {
				doc->displayPages(textOut, 1, doc->getNumPages(), 72, 72, 0, gFalse, gTrue, gFalse);
			}
			delete textOut;
	}
}

void PPdfView::buildJpg()
{
	if(imageList.count()<1) {
		return;
	}
	QString path=QDir::homeDirPath();
	#if defined (Q_OS_WIN32)
		if(path.length()<8) {
			path = "C:/Windows";
		}
	#endif
	if(path.length()>1 && (path.right(1)=="/" || path.right(1)=="\\")) {
		path = path.left(path.length()-1);
	}
	path += "/Desktop";
	path = QDir::convertSeparators(path);
	QFileInfo finf;
	finf.setFile(path);
	if(!finf.exists() || !finf.isDir()) {
		path = qApp->applicationDirPath();
	}
	QString s = QFileDialog::getSaveFileName(path, "Bild-Dateien (*.jpg)", this, "Seite als Bild speichern", "Seite als Bild speichern unter");
	if(!s.isNull() && s!="") {
		if(s.lower().right(4)!=".jpg") { s = s + ".jpg"; }
		QFile f; f.setName(s);
		if(f.exists()) {
			QMessageBox mBox("Speichern unter", "Die Datei existiert bereits. Soll diese berschrieben werden?", 
							 QMessageBox::Warning, QMessageBox::Yes, QMessageBox::Abort | QMessageBox::Default | QMessageBox::Escape, 0);
			mBox.setButtonText(QMessageBox::Yes, "Ja, speichern");
			mBox.setButtonText(QMessageBox::Abort, "Nein, abbrechen");
			int answer = mBox.exec();
			if(answer!=QMessageBox::Yes) {
				return;
			}
		}
		QImage img = *imageList.at(pageCurrent);
		img.save(s, "JPEG", 100);
	}
}

void PPdfView::open()
{
	QString path=QDir::homeDirPath();
	#if defined (Q_OS_WIN32)
		if(path.length()<8) {
			path = "C:/Windows";
		}
	#endif
	if(path.length()>1 && (path.right(1)=="/" || path.right(1)=="\\")) {
		path = path.left(path.length()-1);
	}
	path += "/Desktop";
	path = QDir::convertSeparators(path);
	QFileInfo finf;
	finf.setFile(path);
	if(!finf.exists() || !finf.isDir()) {
		path = qApp->applicationDirPath();
	}

	QString s = QFileDialog::getOpenFileName(path, "PDF-Dateien (*.pdf)", this, "PDF-Datei ffnen", "PDF-Datei auswhlen" );
	if(!s.isNull() && s!="") {
		if(s.lower().right(4)!=".pdf") { s = s + ".pdf"; }
		QFileInfo fi; fi.setFile(s);
		if(fi.exists() && fi.isReadable()) {
			load(fi.absFilePath());
		} else {
			QMessageBox::information( this, "Fehler", "Die Datei:\n"+s+"\nkonnte nicht geffnet werden.", "OK");
		}
	}
}

void PPdfView::saveAs()
{
	if(dataLen==0 || !dataFromFile) {
		return;
	}
	QString path=QDir::homeDirPath();
	#if defined (Q_OS_WIN32)
		if(path.length()<8) {
			path = "C:/Windows";
		}
	#endif
	if(path.length()>1 && (path.right(1)=="/" || path.right(1)=="\\")) {
		path = path.left(path.length()-1);
	}
	path += "/Desktop";
	path = QDir::convertSeparators(path);
	QFileInfo finf;
	finf.setFile(path);
	if(!finf.exists() || !finf.isDir()) {
		path = qApp->applicationDirPath();
	}
	QString s = QFileDialog::getSaveFileName(path, "PDF-Dateien (*.pdf)", this, "PDF-Datei speichern", "Datei speichern unter");
	if(!s.isNull() && s!="") {
		if(s.lower().right(4)!=".pdf") { s = s + ".pdf"; }
		QFile f; f.setName(s);
		if(f.exists()) {
			QMessageBox mBox("Speichern unter", "Die Datei existiert bereits. Soll diese berschrieben werden?", 
							 QMessageBox::Warning, QMessageBox::Yes, QMessageBox::Abort | QMessageBox::Default | QMessageBox::Escape, 0);
			mBox.setButtonText(QMessageBox::Yes, "Ja, speichern");
			mBox.setButtonText(QMessageBox::Abort, "Nein, abbrechen");
			int answer = mBox.exec();
			if(answer!=QMessageBox::Yes) {
				return;
			}
		}
		if(f.open(IO_WriteOnly)) {
			QDataStream streamO(&f);
			streamO.writeRawBytes(dataFromFile, dataLen);
			f.close();
		}
	}
}

void PPdfView::showInSystemViewer()
{
	if(dataLen==0 || !dataFromFile) {
		return;
	}

	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

	// alle temporren Dateien mit temp* lschen
	QString path = QDir::currentDirPath();
	QDir d(path);
	QStringList list = d.entryList("temp*");
	int i,iE=list.count();
	QString tx;
	if(path.mid(path.length()-1) != "/" && path.mid(path.length()-1) != "\\") {
		#if defined (Q_OS_WIN32)
			path += "\\";
		#else
			path += "/";
		#endif
	}
	for(i=0;i<iE;i++) {
		tx = path + *list.at(i);
		QFile::remove(tx);
	}
	// Name fr Temporrdatei festlegen
	QString tempFile = path + "temp.pdf";
	QFile f; f.setName(tempFile);
	i = 0;
	while(f.exists()) {
		i++;
		tempFile = path + "temp" + QString::number(i) + ".pdf";
		f.setName(tempFile);
	}
	// Daten in temporr Datei schreiben
	if(f.open(IO_WriteOnly)) {
		QDataStream streamO(&f);
		streamO.writeRawBytes(dataFromFile, dataLen);
		f.close();
	} else {
		QApplication::restoreOverrideCursor();
		return;
	}
	// festlegen mit welchem Programm die Datei angezeigt werden soll
	QString progr = "";
	{
		//qDebug("Aktueller Pfad fr den Pdf-Viewer: " + QDir::currentDirPath());
		QFile fPr; fPr.setName(path + "ppdfview.ini");
		if(fPr.exists()) {
			if(fPr.open(IO_ReadOnly)) {
				QTextStream ts(&fPr);
				progr = ts.read();
				fPr.close();
			}
		}
	}
	progr = progr.stripWhiteSpace();

	#if defined (Q_OS_WIN32) // Windows

		tempFile.replace("/","\\");
		if(progr=="") { progr = "open"; }
		HINSTANCE result = 0;
		QT_WA( {
			// ShellExecute(Fenster, Operation, File, Parameter, Directory, Anzeige)
			result = ShellExecute(0, progr.ucs2(), tempFile.ucs2(), NULL, QDir::currentDirPath().ucs2(), SW_SHOWNORMAL);
		} , {
			result = ShellExecuteA(0, progr.ascii(), tempFile.ascii(), NULL, QDir::currentDirPath().ascii(), SW_SHOWNORMAL);
		} );
		if((int)result<=32) {
			QApplication::restoreOverrideCursor();
			// Fehlerbehandlung
			QString msg = "Ein PDF Anzeige Programm konnte nicht gestartet werden.";
			msg += "\nBitte tragen Sie in eine Datei 'ppdfview.ini' den Namen des Befehls ein, der in der Shellexecute ausgefhrt werden soll.";
			msg += "\nLegen Sie diese Datei in den Ordner '" + path + "'.";
			QMessageBox::critical( 0, "PDF Anzeige:", msg,"OK" );
		}
	#else

		QProcess *procSys = new QProcess(0);

		#if defined (Q_WS_MACX) // Mac OS X

			if(progr=="") { progr = "/usr/bin/open"; }

		#else // Linux, Unix  // #if defined (Q_WS_X11)

			// Standardpfade durchsuchen
			QFile fPr; fPr.setName(progr);
			if(progr=="" || !fPr.exists()) { 
				progr = "/usr/local/Acrobat13/bin/acroread";
				fPr.setName(progr);
				int i = 14;
				while(!fPr.exists() && i>=0) {
					i--;
					if(i==0) {
						progr = "/usr/local/Acrobat/bin/acroread";
					} else {
						progr = "/usr/local/Acrobat"+QString::number(i)+"/bin/acroread";
					}
					fPr.setName(progr);
				}
				if(!fPr.exists()) {
					progr = "/opt/Acrobat13/bin/acroread";
					fPr.setName(progr);
					int i = 14;
					while(!fPr.exists() && i>=0) {
						i--;
						if(i==0) {
							progr = "/opt/local/Acrobat/bin/acroread";
						} else {
							progr = "/opt/local/Acrobat"+QString::number(i)+"/bin/acroread";
						}
						fPr.setName(progr);
					}
				}
				if(!fPr.exists()) {
					progr = "/usr/local/bin/gs";
					fPr.setName(progr);
				}
				if(!fPr.exists()) {
					progr = "/usr/contrib/bin/gs";
					fPr.setName(progr);
				}
				if(!fPr.exists()) {
					progr = "/usr/local/Acrobat4/bin/acroread";
				}
			}

		#endif

		procSys->addArgument(progr);
		procSys->addArgument(tempFile);

		if(!procSys->start()) {
			QApplication::restoreOverrideCursor();
			// Fehlerbehandlung
			QString msg = "Ein PDF Anzeige Programm konnte nicht gestartet werden.";
			msg += "\nBitte tragen Sie in eine Datei 'ppdfview.ini' den absoluten Pfad und Namen des Programms ein."
			msg += "\nLegen Sie diese Datei in den Ordner '"+ path + "'.";
			QMessageBox::critical( 0, "PDF Anzeige:", msg,"OK" );
			delete procSys;
		}
	#endif

	QApplication::restoreOverrideCursor();
}	

void PPdfView::printWithDialog()
{
	#if defined (Q_OS_WIN32)
		if(reader) {
			reader->dynamicCall("Print()");
			return;
		}
	#endif
	#ifndef QT_NO_PRINTER

		if(dataLen==0 || !dataFromFile) { return; }
		if(imageList.count()<1) { return; }

		QPrinter printer(QPrinter::HighResolution);
		printer.setFullPage(TRUE);
		printer.setPageSize(QPrinter::A4);
		if((*imageList.at(0)).height() < (*imageList.at(0)).width()) {
			printer.setOrientation(QPrinter::Landscape);
		} else {
			printer.setOrientation(QPrinter::Portrait);
		}
		printer.setOptionEnabled(QPrinter::PrintPageRange, true);
		//printer.setOptionEnabled(QPrinter::PrintSelection, true);
		//printer.setPrintRange(QPrinter::Selection);
		printer.setMinMax(1, pageCount);
		printer.setFromTo(1, pageCount);

		if(bColor) { printer.setColorMode(QPrinter::Color); }

		// Druckerdialog im Bildschirm zentrieren (deswegen braucht er ein parentWidget pwidg)
		QWidget* pwidg = new QWidget(0,"", Qt::WStyle_Customize | Qt::WStyle_NoBorder);
		pwidg->setFixedWidth(400);
		pwidg->setFixedHeight(400);
		QRect scr = QApplication::desktop()->screenGeometry();
		pwidg->move(scr.center() - pwidg->rect().center());
	
		if(printer.setup(pwidg)) {

			QPainter painter( &printer );
			if(!painter.isActive()) { // Start zum Drucken verfehlte
				return;
			}

			QPaintDeviceMetrics metrics(painter.device());
			//int dpiX = metrics.logicalDpiX();
			//int dpiY = metrics.logicalDpiY();
			//int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rnder
			int marginLeft = 0;
			int marginTop = 0;
			int marginRight = 0;
			int marginBottom = 0;

			GString* ownerPass, * userPass;
			if(ownerPW!="") {
				ownerPass = new GString(ownerPW.ascii());
			} else {
				ownerPass = NULL;
			}
			if(userPW!="") {
				userPass = new GString(userPW.ascii());
			} else {
				userPass = NULL;
			}
			
			Object obj;
			MemStream* mStr;
			PDFDoc* doc;

			obj.initNull();
			mStr = new MemStream(dataFromFile, 0, dataLen, &obj);
			doc = new PDFDoc(mStr, ownerPass, userPass);

			if(userPass)  { delete userPass; }
			if(ownerPass) { delete ownerPass; }

			if(!doc->isOk()) {

				delete doc;
				Object::memCheck(stderr);
				gMemReport(stderr);

				return;
			}

			SplashColor paperColor;
			SplashOutputDev* splashOut;

			if(bColor) {
				paperColor[0] = paperColor[1] = paperColor[2] = 0xff;
				splashOut = new SplashOutputDev(splashModeRGB8, 1, gFalse, paperColor);
			} else {
				paperColor[0] = 0xff;
				splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);
			}
	
			splashOut->startDoc(doc->getXRef());

			SplashBitmap* bp;
			QImage* image;

			int pageFrom = printer.fromPage();
			int pageTo = printer.toPage();
			// wenn pageTo==0 und pageFrom==0, dann wurde vom Benutzer im Printdialog keine Seitenauswahl getroffen, d.h. alle Seiten sollen gedruckt werden
			if(pageFrom<1) { pageFrom = 1; } // Seitenzahl beginnt mit 1
			if(pageTo<1) { pageTo = pageCount; }
			// wenn vom Benutzer eine zu hohe Seitenendzahl eingegeben wurde
			if(pageTo>pageCount) { pageTo = pageCount; }

			QProgressDialog progress("Die Daten der Seiten werden zum Drucker gesandt ...", "Druck abbrechen", pageTo, this, "Drucken", true);
			progress.setCaption("Drucken");
			progress.setIcon(QPixmap(PPdfViewer::pdfPrint));
			progress.setAutoClose(false);
			progress.setAutoReset(false);

			progress.show();
			progress.setProgress(0);
			qApp->processEvents();

			int pg;
			for(pg=pageFrom;pg<=pageTo;pg++) {

				progress.setProgress(pg);
				progress.setLabelText("Die Daten der "+QString::number(pg)+". Seite werden zum Drucker gesandt ...");
				qApp->processEvents();

				doc->displayPage(splashOut, pg, 300, 300, 0, gFalse, gTrue, gFalse);

				bp = splashOut->getBitmap();

				QByteArray a;
				QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

				SplashColorPtr row;
				row = bp->getDataPtr();

				QString tx = "P5\n";
				if(bColor) { tx = "P6\n"; }
				tx += QString::number(bp->getWidth());
				tx += " ";
				tx += QString::number(bp->getHeight());
				tx += "\n255\n";
				stream->writeRawBytes(tx.ascii(), tx.length());
				if(bColor) {
					stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
				} else {
					stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
				}

				image = new QImage(a);
				
				delete stream;
				a.resetRawData( a.data(), a.size() );

				if(progress.wasCanceled()) {

					printer.abort();
			
					delete splashOut;
					delete doc;
					Object::memCheck(stderr);
					gMemReport(stderr);

					return;
				}

				painter.drawImage(QRect(marginLeft, marginTop, metrics.width()-marginRight,metrics.height()-marginBottom), *image);

				if(pg<pageTo) { printer.newPage(); }

				if(pg==pageTo) {
					QTimer::singleShot(700, &progress, SLOT(close()));
				}
			}

			delete splashOut;
			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);
		}
	#endif
}

QString PPdfView::printData(char* data, uint len, QPrinter* printer, int page, QProgressDialog* progress, bool isColor, QString ownerPassword, QString userPassword)
{
	if(len==0 || !data) { return "Keine Daten zum drucken."; }

	#ifndef QT_NO_PRINTER

		QPainter painter( printer );
		if(!painter.isActive()) { // Start zum Drucken verfehlte
			return "Start zum Drucken verfehlte.";
		}

		QPaintDeviceMetrics metrics(painter.device());
		//int dpiX = metrics.logicalDpiX();
		//int dpiY = metrics.logicalDpiY();
		//int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rnder
		int marginLeft = 0;
		int marginTop = 0;
		int marginRight = 0;
		int marginBottom = 0;

		GString* ownerPass, * userPass;
		if(ownerPW!="") {
			ownerPass = new GString(ownerPW.ascii());
		} else {
			ownerPass = NULL;
		}
		if(userPW!="") {
			userPass = new GString(userPW.ascii());
		} else {
			userPass = NULL;
		}
		
		Object obj;
		MemStream* mStr;
		PDFDoc* doc;

		obj.initNull();
		mStr = new MemStream(data, 0, len, &obj);
		doc = new PDFDoc(mStr, ownerPass, userPass);

		if(userPass)  { delete userPass; }
		if(ownerPass) { delete ownerPass; }

		if(!doc->isOk()) {

			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);

			return "Daten zum Druck konnten nicht gelesen werden.";
		}

		SplashColor paperColor;
		paperColor[0] = 0xff;
		SplashOutputDev* splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);

		splashOut->startDoc(doc->getXRef());

		SplashBitmap* bp;
		QImage* image;

		int pg;
		int firstPage = 1;
		int lastPage = doc->getNumPages();
		if(page>0) {
			if(page>lastPage) {

				delete doc;
				Object::memCheck(stderr);
				gMemReport(stderr);

				return "Das Dokument hat die Seite "+QString::number(page)+" nicht.";
			}
			firstPage = page;
			lastPage = page;
		}

		emit printingPage(0, lastPage);
		qApp->processEvents();

		for(pg=firstPage;pg<=lastPage;pg++) {

			emit printingPage(pg, lastPage);
			qApp->processEvents();

			doc->displayPage(splashOut, pg, 300, 300, 0, gFalse, gTrue, gFalse);

			bp = splashOut->getBitmap();

			QByteArray a;
			QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

			SplashColorPtr row;
			row = bp->getDataPtr();

			QString tx = "P5\n";
			if(isColor) { tx = "P6\n"; }
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n255\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			if(isColor) {
				// hier noch Absturz: Bug!!!
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
			} else {
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
			}

			image = new QImage(a);

			if(progress && progress->wasCanceled()) {
		
				printer->abort();

				delete splashOut;
				delete doc;
				Object::memCheck(stderr);
				gMemReport(stderr);

				return "Druck wurde manuell unterbrochen.";
			}

			painter.drawImage(QRect(marginLeft, marginTop, metrics.width()-marginRight,metrics.height()-marginBottom), *image);

			delete stream;
			a.resetRawData(a.data(), a.size());

			if(pg<lastPage) { printer->newPage(); }
		}

		delete splashOut;
		delete doc;
		Object::memCheck(stderr);
		gMemReport(stderr);
	#endif

	return "";
}

void PPdfView::print(QString fileName, QString ownerPassword, QString userPassword)
{
	#if defined (Q_OS_WIN32)
		if(reader) {
			if(fileName!=fileNamePdf) {
				reader->dynamicCall("LoadFile(const QString&)", fileName);
			}
			reader->dynamicCall("Print()");  // zeigt auch einen Druckdialog an ohne strendes leeres Fenster vom Reader
			return;
		}
	#endif

	char* data;
	uint len = 0;
	QFile f; f.setName(fileName);
	QFileInfo fi; fi.setFile(f);
	if(f.open(IO_ReadOnly)) {
		QDataStream dStr(&f);
		data = new char[fi.size()];
		dStr.readRawBytes(data,fi.size());
		f.close();
		len = fi.size();
		fileNamePdf = fileName;
	}
	if(len==0) {
		return;
	}

	#ifndef QT_NO_PRINTER
		QPrinter printer(QPrinter::HighResolution);
		printer.setFullPage(TRUE);
		printer.setPageSize(QPrinter::A4);
		printer.setOrientation(QPrinter::Portrait);
		printer.setOptionEnabled(QPrinter::PrintPageRange, true);
		printer.setMinMax(1, pageCount);
		printer.setFromTo(1, pageCount);

		int pageFrom = 1; // Seitenzahl beginnt mit 1
		int pageTo = pageCount;

		QPainter painter( &printer );
		if(!painter.isActive()) { // Start zum Drucken verfehlte
			return;
		}

		QProgressDialog progress("Die Daten werden zum Drucker gesandt ...", "Druck abbrechen", pageTo, this, "Drucken", true);
		progress.setCaption("Drucken");
		progress.setIcon(QPixmap(PPdfViewer::pdfPrint));
		progress.setAutoClose(false);
		progress.setAutoReset(false);

		progress.show();
		progress.setProgress(0);
		qApp->processEvents();

		QPaintDeviceMetrics metrics(painter.device());
		//int dpiX = metrics.logicalDpiX();
		//int dpiY = metrics.logicalDpiY();
		//int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rnder
		int marginLeft = 0;
		int marginTop = 0;
		int marginRight = 0;
		int marginBottom = 0;

		GString* ownerPass, * userPass;
		if(ownerPW!="") {
			ownerPass = new GString(ownerPassword.ascii());
		} else {
			ownerPass = NULL;
		}
		if(userPW!="") {
			userPass = new GString(userPassword.ascii());
		} else {
			userPass = NULL;
		}
		
		Object obj;
		MemStream* mStr;
		PDFDoc* doc;

		obj.initNull();
		mStr = new MemStream(dataFromFile, 0, dataLen, &obj);
		doc = new PDFDoc(mStr, ownerPass, userPass);

		if(userPass)  { delete userPass; }
		if(ownerPass) { delete ownerPass; }

		if(!doc->isOk()) {

			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);

			return;
		}

		SplashColor paperColor;
		paperColor[0] = 0xff;
		SplashOutputDev* splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);

		splashOut->startDoc(doc->getXRef());

		SplashBitmap* bp;
		QImage* image;

		int pg;
		for(pg=pageFrom;pg<=pageTo;pg++) {

			progress.setProgress(pg);
			progress.setLabelText("Die Daten der "+QString::number(pg)+". Seite werden zum Drucker gesandt ...");
			qApp->processEvents();

			doc->displayPage(splashOut, pg, 300, 300, 0, gFalse, gTrue, gFalse);

			bp = splashOut->getBitmap();

			QByteArray a;
			QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

			SplashColorPtr row;
			row = bp->getDataPtr();

			QString tx = "P5\n";
			if(bColor) { tx = "P6\n"; }
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n255\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			if(bColor) {
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
			} else {
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
			}

			image = new QImage(a);
			
			delete stream;
			a.resetRawData(a.data(), a.size());

			painter.drawImage(QRect(marginLeft, marginTop, metrics.width()-marginRight,metrics.height()-marginBottom), *image);

			if(pg<pageTo) { printer.newPage(); }

			if(pg==pageTo) {
				QTimer::singleShot(700, &progress, SLOT(close()));
			}
		}

		delete splashOut;
		delete doc;
		Object::memCheck(stderr);
		gMemReport(stderr);
	#endif
}

void PPdfView::print()
{
	if(dataLen==0 || !dataFromFile) { return; }
//	if(imageList.count()<1) { return; }

	#ifndef QT_NO_PRINTER
		QPrinter printer(QPrinter::HighResolution);
		printer.setFullPage(TRUE);
		printer.setPageSize(QPrinter::A4);
//		if((*imageList.at(0)).height() < (*imageList.at(0)).width()) {
//			printer.setOrientation(QPrinter::Landscape);
//		} else {
			printer.setOrientation(QPrinter::Portrait);
//		}
		printer.setOptionEnabled(QPrinter::PrintPageRange, true);
		printer.setMinMax(1, pageCount);
		printer.setFromTo(1, pageCount);

		int pageFrom = 1; // Seitenzahl beginnt mit 1
		int pageTo = pageCount; 

		QPainter painter( &printer );
		if(!painter.isActive()) { // Start zum Drucken verfehlte
			return;
		}

		QProgressDialog progress("Die Daten werden zum Drucker gesandt ...", "Druck abbrechen", pageTo, this, "Drucken", true);
		progress.setCaption("Drucken");
		progress.setIcon(QPixmap(PPdfViewer::pdfPrint));
		progress.setAutoClose(false);
		progress.setAutoReset(false);

		progress.show();
		progress.setProgress(0);
		qApp->processEvents();

		QPaintDeviceMetrics metrics(painter.device());
		//int dpiX = metrics.logicalDpiX();
		//int dpiY = metrics.logicalDpiY();
		//int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rnder
		int marginLeft = 0;
		int marginTop = 0;
		int marginRight = 0;
		int marginBottom = 0;

		GString* ownerPass, * userPass;
		if(ownerPW!="") {
			ownerPass = new GString(ownerPW.ascii());
		} else {
			ownerPass = NULL;
		}
		if(userPW!="") {
			userPass = new GString(userPW.ascii());
		} else {
			userPass = NULL;
		}
		
		Object obj;
		MemStream* mStr;
		PDFDoc* doc;

		obj.initNull();
		mStr = new MemStream(dataFromFile, 0, dataLen, &obj);
		doc = new PDFDoc(mStr, ownerPass, userPass);

		if(userPass)  { delete userPass; }
		if(ownerPass) { delete ownerPass; }

		if(!doc->isOk()) {

			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);

			return;
		}

		SplashColor paperColor;
		paperColor[0] = 0xff;
		SplashOutputDev* splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);

		splashOut->startDoc(doc->getXRef());

		SplashBitmap* bp;
		QImage* image;

		int pg;

		for(pg=pageFrom;pg<=pageTo;pg++) {

			progress.setProgress(pg);
			progress.setLabelText("Die Daten der "+QString::number(pg)+". Seite werden zum Drucker gesandt ...");
			qApp->processEvents();

			doc->displayPage(splashOut, pg, 300, 300, 0, gFalse, gTrue, gFalse);

			bp = splashOut->getBitmap();

			QByteArray a;
			QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

			SplashColorPtr row;
			row = bp->getDataPtr();

			QString tx = "P5\n";
			if(bColor) { tx = "P6\n"; }
			tx += QString::number(bp->getWidth());
			tx += " ";
			tx += QString::number(bp->getHeight());
			tx += "\n255\n";
			stream->writeRawBytes(tx.ascii(), tx.length());
			if(bColor) {
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
			} else {
				stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
			}

			image = new QImage(a);

			if(progress.wasCanceled()) {
		
				printer.abort();

				delete splashOut;
				delete doc;
				Object::memCheck(stderr);
				gMemReport(stderr);

				return;
			}

			painter.drawImage(QRect(marginLeft, marginTop, metrics.width()-marginRight,metrics.height()-marginBottom), *image);

			delete stream;
			a.resetRawData(a.data(), a.size());

			if(pg<pageTo) { printer.newPage(); }

			if(pg==pageTo) {
				QTimer::singleShot(700, &progress, SLOT(close()));
			}
		}

		delete splashOut;
		delete doc;
		Object::memCheck(stderr);
		gMemReport(stderr);
	#endif
}

void PPdfView::printCurrentPage()
{
	if(dataLen==0 || !dataFromFile) { return; }
	if(imageList.count()<(unsigned int)pageCurrent) { return; }

	#ifndef QT_NO_PRINTER
		QPrinter printer(QPrinter::HighResolution);
		printer.setFullPage(TRUE);
		printer.setPageSize(QPrinter::A4);
		if((*imageList.at(pageCurrent-1)).height() < (*imageList.at(pageCurrent-1)).width()) {
			printer.setOrientation(QPrinter::Landscape);
		} else {
			printer.setOrientation(QPrinter::Portrait);
		}
		printer.setOptionEnabled(QPrinter::PrintPageRange, true);
		printer.setMinMax(1, pageCount);
		printer.setFromTo(1, pageCount);

		if(pageCurrent<1) { return; }
		if(pageCurrent>pageCount) { return; }

		QPainter painter( &printer );
		if(!painter.isActive()) { // Start zum Drucken verfehlte
			return;
		}

		QProgressDialog progress("Die Daten der " + QString::number(pageCurrent) + ". Seite werden zum Drucker gesandt ...", "Druck abbrechen", 1, this, "Drucken", true);
		progress.setCaption("Drucken");
		progress.setIcon(QPixmap(PPdfViewer::pdfPrint));
		progress.setAutoClose(false);
		progress.setAutoReset(false);

		progress.show();
		progress.setProgress(0);
		qApp->processEvents();

		QPaintDeviceMetrics metrics(painter.device());
		//int dpiX = metrics.logicalDpiX();
		//int dpiY = metrics.logicalDpiY();
		//int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rnder
		int marginLeft = 0;
		int marginTop = 0;
		int marginRight = 0;
		int marginBottom = 0;

		GString* ownerPass, * userPass;
		if(ownerPW!="") {
			ownerPass = new GString(ownerPW.ascii());
		} else {
			ownerPass = NULL;
		}
		if(userPW!="") {
			userPass = new GString(userPW.ascii());
		} else {
			userPass = NULL;
		}
		
		Object obj;
		MemStream* mStr;
		PDFDoc* doc;

		obj.initNull();
		mStr = new MemStream(dataFromFile, 0, dataLen, &obj);
		doc = new PDFDoc(mStr, ownerPass, userPass);

		if(userPass)  { delete userPass; }
		if(ownerPass) { delete ownerPass; }

		if(!doc->isOk()) {

			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);

			return;
		}

		SplashColor paperColor;
		paperColor[0] = 0xff;
		SplashOutputDev* splashOut = new SplashOutputDev(splashModeMono8, 1, gFalse, paperColor);

		splashOut->startDoc(doc->getXRef());

		SplashBitmap* bp;
		QImage* image;

		doc->displayPage(splashOut, pageCurrent, 300, 300, 0, gFalse, gTrue, gFalse);

		bp = splashOut->getBitmap();

		QByteArray a;
		QDataStream* stream = new QDataStream(a, IO_WriteOnly | IO_Translate);

		SplashColorPtr row;
		row = bp->getDataPtr();

		QString tx = "P5\n";
		if(bColor) { tx = "P6\n"; }
		tx += QString::number(bp->getWidth());
		tx += " ";
		tx += QString::number(bp->getHeight());
		tx += "\n255\n";
		stream->writeRawBytes(tx.ascii(), tx.length());
		if(bColor) {
			stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight()*3);
		} else {
			stream->writeRawBytes((const char*)row, bp->getWidth()*bp->getHeight());
		}

		image = new QImage(a);
		
		delete stream;
		a.resetRawData(a.data(), a.size());

		if(progress.wasCanceled()) {
	
			printer.abort();

			delete splashOut;
			delete doc;
			Object::memCheck(stderr);
			gMemReport(stderr);

			return;
		}

		progress.setProgress(1);
		qApp->processEvents();

		painter.drawImage(QRect(marginLeft, marginTop, metrics.width()-marginRight,metrics.height()-marginBottom), *image);

		QTimer::singleShot(700, &progress, SLOT(close()));

		delete splashOut;
		delete doc;
		Object::memCheck(stderr);
		gMemReport(stderr);
	#endif
}

void PPdfView::printWithDialog(QString fileName, QString ownerPassword, QString userPassword)
{
	#if defined (Q_OS_WIN32)
		if(!reader) {
			if(bWithAdobe==false) {
				bWithAdobe = true;
				init();
				bWithAdobe = false;
			} else {
				init();
			}
		}
		if(!reader) { return; }
		reader->dynamicCall("LoadFile(const QString&)", fileName);
		reader->dynamicCall("Print()");
	#endif
}


void PPdfView::dragEnterEvent(QDragEnterEvent* event)
{
	QStringList list;
	if(QUriDrag::decodeLocalFiles(event, list)) {
		int c = list.count();
		if(c==0) { return; }
		QString name = *list.at(0);
		//qDebug("name: "+name);
		if(name.isNull() || name=="") { return; }
		QFileInfo fi(name);
		if(fi.isSymLink()) { return; }
		if(fi.isDir()) { return; }
		if(fi.isFile()) {
			if(name.lower().right(4)==".pdf") {
				event->accept();
			}
		}
	}
}

void PPdfView::dropEvent(QDropEvent* event)
{
	QStringList list;
	if(QUriDrag::decodeLocalFiles(event, list)) {
		if(list.count()>0) {
			emit droppedFileName(*list.at(0));
		}
	}
}

PPdfViewer::PPdfViewer(QWidget* parent, const char* name, WFlags f) : QVBox(parent, name, f)
{
	setCaption(" PDF Viewer");
	setIcon(QPixmap(pdfCaption));

	if(QColor::numBitPlanes()>8) setPaletteBackgroundColor(QColor::QColor(200,195,195));

	QToolTip::setGloballyEnabled(true);
	QToolTip::setWakeUpDelay(0);

	hbTop = new QHBox(this);
	hbTop->setFrameStyle(QFrame::Plain);
	hbTop->setMargin(1);
	hbTop->setSpacing(2);
	hbTop->setFixedHeight(30);
	if(QColor::numBitPlanes()>8) hbTop->setPaletteBackgroundColor(QColor::QColor(200,195,195));

	QWidget* distance = new QWidget(hbTop);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(10);

	tbFileOpen = new QToolButton(hbTop);
	tbFileOpen->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbFileOpen->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbFileOpen->setIconSet(QPixmap(pdfFileOpen));
	tbFileOpen->setFixedWidth(30);
	tbFileOpen->setAutoRaise(true);
	QToolTip::add(tbFileOpen, "PDF-Datei ffnen");

	tbFileSaveAs = new QToolButton(hbTop);
	tbFileSaveAs->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbFileSaveAs->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbFileSaveAs->setIconSet(QPixmap(pdfFileSaveAs));
	tbFileSaveAs->setFixedWidth(30);
	tbFileSaveAs->setAutoRaise(true);
	QToolTip::add(tbFileSaveAs, "Speichern unter");

	tbPrevPage = new PToolButtonRep(hbTop);
	tbPrevPage->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbPrevPage->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbPrevPage->setIconSet(QPixmap(pdfPageUp));
	tbPrevPage->setFixedWidth(30);
	tbPrevPage->setAutoRaise(true);
	QToolTip::add(tbPrevPage, "Seite zurck");

	tbNextPage = new PToolButtonRep(hbTop);
	tbNextPage->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbNextPage->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbNextPage->setIconSet(QPixmap(pdfPageDown));
	tbNextPage->setFixedWidth(30);
	tbNextPage->setAutoRaise(true);
	QToolTip::add(tbNextPage, "Seite vorwrts");

	distance = new QWidget(hbTop);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	lblPage = new QLabel(hbTop);
	lblPage->setAlignment(Qt::AlignCenter);
	lblPage->setFrameStyle(QFrame::Panel | QFrame::Plain);
	lblPage->setFixedHeight(22);

	distance = new QWidget(hbTop);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	tbFitH = new QToolButton(hbTop);
	tbFitH->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbFitH->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbFitH->setIconSet(QPixmap(pdfFitH));
	tbFitH->setFixedWidth(30);
	tbFitH->setAutoRaise(true);
	QToolTip::add(tbFitH, "Breite anpassen");

	tbFitHV = new QToolButton(hbTop);
	tbFitHV->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbFitHV->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbFitHV->setIconSet(QPixmap(pdfFitHV));
	tbFitHV->setFixedWidth(30);
	tbFitHV->setAutoRaise(true);
	QToolTip::add(tbFitHV, "Grsse anpassen");

	tbColor = new QToolButton(hbTop);
	tbColor->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbColor->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbColor->setIconSet(QPixmap(pdfColor));
	tbColor->setFixedWidth(30);
	tbColor->setAutoRaise(true);
	QToolTip::add(tbColor, "In Farbe anzeigen");

	tbText = new QToolButton(hbTop);
	tbText->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbText->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbText->setIconSet(QPixmap(pdfText));
	tbText->setFixedWidth(30);
	tbText->setAutoRaise(true);
	QToolTip::add(tbText, "Text von markierter Seite extrahieren");

	tbJpg = new QToolButton(hbTop);
	tbJpg->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbJpg->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbJpg->setIconSet(QPixmap(pdfJpg));
	tbJpg->setFixedWidth(30);
	tbJpg->setAutoRaise(true);
	QToolTip::add(tbJpg, "JPG-Bild von markierter Seite erstellen");

	tbPrint = new QToolButton(hbTop);
	tbPrint->setCursor(QCursor(Qt::PointingHandCursor));
	if(QColor::numBitPlanes()>8) tbPrint->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	tbPrint->setTextLabel("",false);
	tbPrint->setPixmap(QPixmap(pdfPrint));
	tbPrint->setFixedWidth(30);
	tbPrint->setAutoRaise(true);
	QToolTip::add(tbPrint, "Drucken");

	distance = new QWidget(hbTop);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));

	view = new PPdfView(this);

	view->setLineWidth(1);
	view->setFrameStyle(QFrame::Panel | QFrame::Sunken);//QFrame::Sunken NoFrame

	if(QColor::numBitPlanes()>8) view->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	if(QColor::numBitPlanes()>8) view->viewport()->setPaletteBackgroundColor(QColor::QColor(215,210,210));
	view->verticalScrollBar()->setPaletteBackgroundColor(QColor::QColor(210,205,205));
	QPalette pal = view->verticalScrollBar()->palette();
	pal.setColor(QColorGroup::Button, QColor(200,195,195));
	view->verticalScrollBar()->setPalette(pal);

	view->hbTop = hbTop;

	QObject::connect(tbFileOpen,   SIGNAL(clicked()),    view, SLOT(open()));
	QObject::connect(tbFileSaveAs, SIGNAL(clicked()),    view, SLOT(saveAs()));
	QObject::connect(tbPrevPage,   SIGNAL(downHolded()), view, SLOT(gotoPrevPage()));
	QObject::connect(tbNextPage,   SIGNAL(downHolded()), view, SLOT(gotoNextPage()));
	QObject::connect(tbFitH,       SIGNAL(clicked()),    view, SLOT(setSizeFitH()));
	QObject::connect(tbFitHV,      SIGNAL(clicked()),    view, SLOT(setSizeFitHV()));
	QObject::connect(tbColor,      SIGNAL(clicked()),    this, SLOT(setColor()));
	QObject::connect(tbText,       SIGNAL(clicked()),    view, SLOT(extractText()));
	QObject::connect(tbJpg,        SIGNAL(clicked()),    view, SLOT(buildJpg()));
	QObject::connect(tbPrint,      SIGNAL(clicked()),    view, SLOT(printWithDialog()));

	QObject::connect(view, SIGNAL(pageChanged(const QString&)), lblPage, SLOT(setText(const QString&)));
	QObject::connect(view, SIGNAL(pageChanged(const QString&)), this, SLOT(setPageLabelWidth(const QString&)));

	hbBottom = new QHBox(this);
	hbBottom->setFrameStyle(QFrame::Plain);
	hbBottom->setMargin(2);
	hbBottom->setSpacing(2);

	hbBottom->setFixedHeight(36);
	if(QColor::numBitPlanes()>8) hbBottom->setPaletteBackgroundColor(QColor::QColor(200,195,195));

	distance = new QWidget(hbBottom);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	pbPrintWithDialog = new PButton("Drucken (mit Dialog)", hbBottom);

	distance = new QWidget(hbBottom);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	pbPrint = new PButton("Direktdruck (alle Seiten)", hbBottom);

	distance = new QWidget(hbBottom);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	pbPrintCurrentPage = new PButton("Direktdruck (markierte Seite)", hbBottom);

	distance = new QWidget(hbBottom);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	pbShowInSystem = new PButton(QPixmap(pdfSymbol), "", hbBottom);
	pbShowInSystem->setFixedWidth(40);
	pbShowInSystem->setFixedHeight(30);

	distance = new QWidget(hbBottom);
	if(QColor::numBitPlanes()>8) distance->setPaletteBackgroundColor(QColor::QColor(200,195,195));
	distance->setFixedWidth(0);

	QObject::connect(pbPrint, SIGNAL(clicked()), view, SLOT(print()));
	QObject::connect(pbPrintWithDialog, SIGNAL(clicked()), view, SLOT(printWithDialog()));
	QObject::connect(pbPrintCurrentPage, SIGNAL(clicked()), view, SLOT(printCurrentPage()));
	
	QObject::connect(pbShowInSystem, SIGNAL(clicked()), view, SLOT(showInSystemViewer()));

	pbPrint->setFixedHeight(pbShowInSystem->height());
	pbPrintWithDialog->setFixedHeight(pbShowInSystem->height());
	pbPrintCurrentPage->setFixedHeight(pbShowInSystem->height());

	QObject::connect(view, SIGNAL(droppedFileName(const QString&)), this, SLOT(setCaptionText(const QString&)));

	QObject::connect(view, SIGNAL(sendPageCount(int)), this, SLOT(updateButtons(int)));
}

void PPdfViewer::setCaptionText(const QString& capt)
{
	QFileInfo fi; fi.setFile(capt);
	if(fi.exists()) {
		setCaption(" " + fi.fileName() + "  - PDF Viewer");
	}
}

PPdfViewer::~PPdfViewer()
{
	if(lblPage) delete lblPage;

	if(tbFileOpen) delete tbFileOpen;
	if(tbFileSaveAs) delete tbFileSaveAs;	
	if(tbPrevPage) delete tbPrevPage;
	if(tbNextPage) delete tbNextPage;
	if(tbFitH) delete tbFitH;
	if(tbFitHV) delete tbFitHV;
	if(tbColor) delete tbColor;
	if(tbText) delete tbText;
	if(tbJpg) delete tbJpg;
	if(tbPrint) delete tbPrint;

	if(pbPrint) delete pbPrint;
	if(pbPrintWithDialog) delete pbPrintWithDialog;
	if(pbPrintCurrentPage) delete pbPrintCurrentPage;
	if(pbShowInSystem) delete pbShowInSystem;

//	if(hbTop) delete hbTop;  // Absturz, weil schon im PdfView delete stattfindet

	if(hbBottom) delete hbBottom; // muss am Ende stehen, sonst Absturz
	if(view) delete view; // muss am Ende stehen, sonst Absturz
}

void PPdfViewer::updateButtons(int n)
{
	if(n<=1) {
		pbPrintCurrentPage->hide();
		pbPrint->setText("Direktdruck");
	} else {
		pbPrintCurrentPage->show();
		pbPrint->setText("Direktdruck (alle Seiten)");
	}
}

void PPdfViewer::reduceButtons()
{
	if(tbFileOpen) tbFileOpen->hide();
	if(tbText) tbText->hide();
	if(tbJpg)  tbJpg->hide();
}

PPdfView* PPdfViewer::pdfView()
{
	return view;
}

void PPdfViewer::showToolBarTop(bool b)
{
	if(b) {
		// Toolbar oben
		hbTop->show();
	} else {
		hbTop->hide();
	}
}

void PPdfViewer::showToolBarBottom(bool b)
{
	if(b) {
		// Toolbar unten
		hbBottom->show();
	} else {
		hbBottom->hide();
	}
}

void PPdfViewer::setColor()
{
	if(view->bColor==false) {
		view->bColor = true;
		tbColor->setIconSet(QPixmap(pdfGrayScale));
		QToolTip::add(tbColor, "In Grauskala anzeigen");
	} else {
		view->bColor = false;
		tbColor->setIconSet(QPixmap(pdfColor));
		QToolTip::add(tbColor, "In Farbe anzeigen");
	}
	view->loadData();
}

void PPdfViewer::setPageLabelWidth(const QString& tx)
{
	if(!lblPage) { return; }
	QFontMetrics fm(lblPage->font());
	lblPage->setFixedWidth(fm.width(tx));
}

void PPdfViewer::keyPressEvent(QKeyEvent* ev)
{
	if(ev->key()==Qt::Key_Up) {
		int contentV = view->contentsY()-18;
		if(contentV<0) { contentV=0; }
		view->setContentsPos(view->contentsX(), contentV);
		view->updateView();
	}
	if(ev->key()==Qt::Key_Down) {
		int contentV = view->contentsY()+18;
		if(contentV>view->contentsHeight()) { contentV = view->contentsHeight(); }
		view->setContentsPos(view->contentsX(), contentV);
		view->updateView();
	}
	if(ev->key()==Qt::Key_PageUp) {
		view->gotoPrevPage();
	}
	if(ev->key()==Qt::Key_PageDown) {
		view->gotoNextPage();
	}
	if(ev->key()==Qt::Key_Home) {
		view->gotoPage(1);
	}
	if(ev->key()==Qt::Key_End) {
		view->setContentsPos(view->contentsX(), view->contentsHeight() - view->viewport()->height());
		view->updateView();
		//QTimer::singleShot(10, view, SLOT(updateView()));
	}

	QVBox::keyPressEvent(ev);
}
