#include "pxml_formatparser.h"

PXML_FormatParser::PXML_FormatParser() : QXmlDefaultHandler()
{
	data = "";
	dataInstruction = "";
	dataStartPrefix = "";
	countTabs = 0;
	countAttributes = 0;
}

void PXML_FormatParser::reset()
{
	data = "";
	dataInstruction = "";
	dataStartPrefix = "";
	countTabs = 0;
	countAttributes = 0;
}

QString PXML_FormatParser::getDataInstruction()
{
	return dataInstruction;
}

QString PXML_FormatParser::getDataStartPrefix()
{
	return dataStartPrefix;
}

QString PXML_FormatParser::getFormattedString()
{
	return data.stripWhiteSpace();
}

bool PXML_FormatParser::startElement(const QString& namespaceURI,
                                   const QString& localName,
                                   const QString& qName,
                                   const QXmlAttributes& attributes)
{
	QString txTab(QChar::QChar(9));
	QString txR(QChar::QChar(10));

	countAttributes = attributes.length();

	if(countAttributes < 1) {
		if(data=="") {

			data += "<"+qName+">" + txTab + txR;

			countTabs++;
		} else {
			QString str;
			str.fill(QChar::QChar(9), countTabs);

			data += str + "<"+qName+">" + txTab + txR;

			countTabs++;
		}
	}

    if(countAttributes > 0) {

		if(data=="") {

			data += "<" + qName + txTab + txR;

			countTabs++;
		} else {
			QString str;
			str.fill(QChar::QChar(9), countTabs);

			data += str + "<" + qName + txTab + txR;

			countTabs++;
		}

		for(int i=0; i<countAttributes; i++) {

			QString str;
			str.fill(QChar::QChar(9), countTabs);

			QString text = attributes.value(i);

			QString txR(QChar::QChar(10));
			QString txTab(QChar::QChar(9));
			QString txR2(QChar::QChar(13));
			text.replace(txR," ");
			text.replace(txR2," ");
			text.replace(txTab," ");
			text = text.stripWhiteSpace();

			QString name = attributes.qName(i);

			if(i<countAttributes-1) {
				data += str + name + "=\"" + text + "\"" + txR;
			}
			if(i==countAttributes-1) {
				data += str + name + "=\"" + text + "\">" + txR;
			}
		}
    }

    return true;
}

bool PXML_FormatParser::characters(const QString& ch)
{
	QString text(ch);

	QString txR(QChar::QChar(10));
	QString txTab(QChar::QChar(9));
	QString txR2(QChar::QChar(13));
	text.replace(txR," ");
	text.replace(txR2," ");
	text.replace(txTab," ");
	text = text.simplifyWhiteSpace();

	if(text!="" && data!="") {
		if(countAttributes>0) {
			int p = data.findRev(txR);
			if(p>0) {
				QString leftPart = data.mid(0, p);
				QString rightPart = data.mid(p, data.length() - p);
				data = leftPart.stripWhiteSpace() + text + rightPart;
			}
		} else {
			data = data.stripWhiteSpace();
			data += text;
		}
	}

	return true;
}

bool PXML_FormatParser::endElement(const QString & namespaceURI, const QString & localName, const QString & qName)
{
	countTabs--;

	QString txTab(QChar::QChar(9));
	QString txR(QChar::QChar(10));

	if(data=="") {

		data += "</"+qName+">" + txTab + txR;
	} else {

		QString eStr = data.right(data.length() - data.findRev("</"));

		if(eStr.contains("<"+qName)) {
			data = data.stripWhiteSpace();
			data += "</"+qName+">" + txTab + txR;
		} else {
			QString str="";
			str.fill(QChar::QChar(9), countTabs);
			data += str + "</"+qName+">" + txTab + txR;
		}
	}

    return true;
}

void PXML_FormatParser::setDocumentLocator(QXmlLocator*) //(QXmlLocator* locator)
{
}

bool PXML_FormatParser::startDocument()
{
	return true;
}

bool PXML_FormatParser::endDocument()
{
	return true;
}

bool PXML_FormatParser::startPrefixMapping(const QString& prefix, const QString& uri)
{
	//qDebug("startPrefixMapping prefix: "+prefix);
	//qDebug("startPrefixMapping uri: "+uri);
	if(dataStartPrefix=="") { dataStartPrefix =  prefix; }
	return true;
}

bool PXML_FormatParser::endPrefixMapping(const QString& prefix) //(const QString& prefix)
{
	return true;
}

bool PXML_FormatParser::ignorableWhitespace(const QString& ch) //(const QString& ch)
{
	//qDebug("ignorableWhitespace: "+ch);
	return true;
}

bool PXML_FormatParser::processingInstruction(const QString& target, const QString& dataStr)
{
	//qDebug("processingInstruction target: "+target);
	//qDebug("processingInstruction data: "+data);

	dataInstruction = "<?" + target + " " + dataStr + " ?>";

	return true;
}

bool PXML_FormatParser::skippedEntity(const QString& name) //(const QString& name)
{
	//qDebug("skippedEntity: "+name);

	return true;
}

QString PXML_FormatParser::errorString()
{
	return "";
} 
