#include "pxmlvalidator.h"
#include "pxml_formatparser.h"

#include <qfile.h>
#include <qfileinfo.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qtooltip.h>
#include <qfiledialog.h>
#include <qdragobject.h>
#include <qmessagebox.h>
#include <qcursor.h>
#include <qapplication.h>

#include <qsciprinter.h>

#include "qscilexerxml.h"

#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/util/OutOfMemoryException.hpp>

#include "StdInParseHandlers.hpp"

extern QString StdInParseERROR;

using namespace xercesc;

/* XPM */
static const char* redoXPM[] = {
// columns rows colors chars-per-pixel 
"22 22 3 1",
"n c #7f7f7f",
"X c #00007f",
". c None",
// pixels 
"......................",
"......................",
"......................",
"......................",
"......................",
"....nXXXXXX...........",
"...XXXXXXXXXX......X..",
"..nXX......XXXX...XX..",
"..XX.........XXX.XXX..",
"..XX..........XXXXXX..",
"..XX...........XXXXX..",
"..XX..........XXXXXX..",
"..XX.........XXXXXXX..",
"..nXX.......XXXXXXXX..",
"...XXXn...............",
"....XXX...............",
"......................",
"......................",
"......................",
"......................",
"......................",
"......................"
};

/* XPM */
static const char* undoXPM[] = {
// columns rows colors chars-per-pixel
"22 22 3 1",
"n c #7f7f7f",
"X c #00007f",
". c None",
// pixels
"......................",
"......................",
"......................",
"......................",
"......................",
"...........XXXXXXn....",
"..X......XXXXXXXXXX...",
"..XX...XXXX......XXn..",
"..XXX.XXX.........XX..",
"..XXXXXX..........XX..",
"..XXXXX...........XX..",
"..XXXXXX..........XX..",
"..XXXXXXX.........XX..",
"..XXXXXXXX.......XXn..",
"...............nXXX...",
"...............XXX....",
"......................",
"......................",
"......................",
"......................",
"......................",
"......................"
};

/* XPM */
static const char *fileopen[] = {
"16 13 5 1",
". c #040404",
"# c #808304",
"a c None",
"b c #f3f704",
"c c #f3f7f3",
"aaaaaaaaa...aaaa",
"aaaaaaaa.aaa.a.a",
"aaaaaaaaaaaaa..a",
"a...aaaaaaaa...a",
".bcb.......aaaaa",
".cbcbcbcbc.aaaaa",
".bcbcbcbcb.aaaaa",
".cbcb...........",
".bcb.#########.a",
".cb.#########.aa",
".b.#########.aaa",
"..#########.aaaa",
"...........aaaaa"
};

/* XPM */
static const char *fileprint[] = {
"16 14 6 1",
". c #000000",
"# c #848284",
"a c #c6c3c6",
"b c #ffff00",
"c c #ffffff",
"d c None",
"ddddd.........dd",
"dddd.cccccccc.dd",
"dddd.c.....c.ddd",
"ddd.cccccccc.ddd",
"ddd.c.....c....d",
"dd.cccccccc.a.a.",
"d..........a.a..",
".aaaaaaaaaa.a.a.",
".............aa.",
".aaaaaa###aa.a.d",
".aaaaaabbbaa...d",
".............a.d",
"d.aaaaaaaaa.a.dd",
"dd...........ddd"
};

/* XPM */
static const char *filesave[] = {
"14 14 4 1",
". c #040404",
"# c #808304",
"a c #bfc2bf",
"b c None",
"..............",
".#.aaaaaaaa.a.",
".#.aaaaaaaa...",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".##........##.",
".############.",
".##.........#.",
".##......aa.#.",
".##......aa.#.",
".##......aa.#.",
"b............."
};

/* XPM */
static const char *valid[] = {
"15 12 103 2",
"#m c #0a8608",
"#v c #0a8707",
"#a c #0b8709",
".Z c #0c850a",
".L c #0c850b",
".z c #0d840c",
"#C c #0d8907",
".8 c #128715",
".o c #147e16",
"#G c #157d17",
".7 c #1c9f0c",
"#A c #1fa40e",
"#u c #21a90e",
"#t c #22a50f",
"#l c #22a80e",
"## c #22a90e",
"#B c #22ab0f",
".h c #238628",
".e c #239a27",
".K c #23a90f",
".Y c #23aa0e",
".S c #23ab0f",
".k c #24992a",
".y c #24ab0f",
"#r c #25a610",
"#k c #27a811",
".n c #27ab10",
".m c #28a90f",
".x c #28a911",
".X c #28a913",
"#s c #28aa11",
".b c #29962e",
".v c #29992e",
"#. c #29aa13",
"#g c #2aaa13",
".J c #2b9a32",
"#i c #2bab12",
"#z c #2c8933",
".a c #2d9c33",
"#q c #2e8e36",
".V c #2e9b36",
"#f c #309438",
"#h c #30ae15",
".4 c #34b018",
"#F c #36833f",
".6 c #36b118",
".5 c #38b319",
".P c #3cae32",
".R c #3cb418",
".g c #3cb522",
"#j c #42b52b",
".E c #43b52c",
".T c #499a55",
".3 c #4aab4e",
".D c #4cbe3a",
"#w c #4d695a",
"#D c #4d6a5a",
".0 c #4e655b",
"#b c #4e675b",
"#n c #4e685b",
".M c #4f655c",
"#H c #4f6a5c",
".F c #4fad59",
".A c #50645d",
".Q c #5ac238",
"#x c #5e656e",
"#c c #606770",
".1 c #616871",
".N c #626972",
".9 c #66c157",
".W c #66c256",
".w c #67c458",
".l c #6ac559",
".f c #6dc75b",
".B c #767e89",
"#K c #777f8a",
".p c #7b838f",
"#J c #848d9a",
".C c #8ed19d",
"#E c #96a1af",
"#o c #99a3b2",
"#d c #9ba5b4",
".2 c #9ba6b5",
".r c #9cd6b1",
".O c #a0abba",
".c c #a3c1be",
".i c #aab6c6",
"#y c #abbcc7",
"#p c #aebfcb",
".U c #b2c4cf",
"#I c #b3c0d1",
".q c #b8e4d2",
".G c #c0cee0",
"#e c #c0d3e0",
".# c #c3dfe3",
".d c #c6e3e7",
".j c #c8e3e9",
".u c #cbe4ec",
".I c #cce4ee",
".s c #d2e1f4",
".H c #d6e4f9",
".t c #d8e7fc",
"Qt c None", // #ffffff",
"QtQtQtQtQtQtQtQtQtQt.#.a.b.cQt",
"QtQtQtQtQtQtQtQtQt.d.e.f.g.h.i",
"QtQtQtQtQtQtQtQt.j.k.l.m.n.o.p",
"Qt.q.r.s.tQtQt.u.v.w.x.y.z.A.B",
".C.D.E.F.G.H.I.J.w.x.K.L.M.N.O",
".P.Q.R.S.T.U.V.W.X.Y.Z.0.1.2Qt",
".3.4.5.6.7.8.9#.###a#b#c#dQtQt",
"#e#f#g#h#i#j#k#l#m#n#c#oQtQtQt",
"Qt#p#q#r#s#t#u#v#w#x#oQtQtQtQt",
"QtQt#y#z#A#B#C#D#x#EQtQtQtQtQt",
"QtQtQt#y#F#G#H#x#EQtQtQtQtQtQt",
"QtQtQtQt#I#J#K#dQtQtQtQtQtQtQt"};

/* XPM */
static const char *invalid[] = {
"15 15 180 2",
"aD c #606770",
"#M c #626972",
"#x c #636a73",
"aW c #6b545c",
"aN c #6d5359",
"aq c #6d555b",
"aC c #6f5258",
"#i c #717883",
"aX c #717984",
"aO c #9aa4b3",
".5 c #9ca7b6",
"aE c #9da8b7",
"aV c #a41611",
"aP c #a7828b",
"aU c #ae1710",
"aT c #ae190f",
"aF c #ae848b",
"aS c #b01a0e",
"aQ c #b11b11",
"ac c #b11d10",
".Q c #b18084",
"aR c #b31b10",
"ar c #b3868d",
".C c #bc7978",
".q c #bf807c",
"aM c #c1200a",
"#0 c #c4210d",
"ap c #c7230d",
"aB c #c92911",
"aG c #ca2810",
"ad c #ca9ba2",
"#L c #cb250c",
".4 c #cc280e",
"#h c #cf2d13",
"#w c #cf2f16",
"#1 c #d02910",
"as c #d03820",
".g c #d29b9c",
"ae c #d53418",
"#N c #d92c10",
"#y c #db2e0f",
"#j c #df3210",
"aA c #e13d15",
"ao c #e4431d",
".6 c #e53611",
".f c #e53612",
".r c #e5a9a8",
".D c #e5aaaa",
".h c #e5abab",
".# c #e5acac",
".R c #e73613",
".P c #e83e18",
"at c #e9431a",
"an c #e94a21",
"az c #ea4014",
"af c #ea4920",
"#2 c #ea4a20",
"aa c #ea542f",
".e c #eb3a13",
".B c #eb4822",
".d c #ed3c15",
"aH c #ee4316",
"#O c #ee532c",
".c c #ef3d15",
".a c #f03d14",
".b c #f03f15",
"aL c #f04010",
".p c #f0441d",
"#z c #f0603a",
"am c #f14e23",
"a# c #f1603c",
"#k c #f16946",
".s c #f16949",
"#Y c #f16b48",
"ab c #f24419",
"au c #f2491b",
".E c #f25833",
"aK c #f34112",
"aJ c #f34314",
"ay c #f34315",
"aI c #f34516",
"#Z c #f3562d",
".i c #f35834",
"#K c #f36a47",
".7 c #f37251",
".m c #f44a1e",
"ag c #f45327",
".O c #f45328",
".F c #f47454",
".n c #f54a1d",
".l c #f54c20",
".A c #f54d21",
".k c #f55227",
".3 c #f55a32",
"#3 c #f55d31",
".j c #f55e34",
".t c #f56b47",
".S c #f57656",
".T c #f57a56",
"#v c #f57a59",
"#J c #f58061",
"ax c #f64617",
".G c #f6704c",
"#g c #f67452",
"aw c #f74919",
"av c #f74a1b",
".o c #f74e21",
"al c #f75124",
".v c #f76236",
".u c #f7663d",
"#P c #f76f47",
"#X c #f77c5c",
".8 c #f78463",
"#u c #f78667",
".y c #f85627",
".x c #f8592b",
".2 c #f8643c",
"a. c #f86741",
"#f c #f87a58",
"#A c #f87e5a",
".U c #f87f5d",
"#l c #f88565",
".z c #f95325",
"ah c #f9582c",
".N c #f95a2e",
".w c #f95c2f",
"#4 c #f96940",
".H c #f9704a",
"ak c #fa5325",
".I c #fa7047",
"#Q c #fa7f5a",
".9 c #fa8e6d",
"#I c #fa9276",
"aj c #fb5527",
"ai c #fb572b",
".M c #fb6136",
"#9 c #fb6941",
"#5 c #fb6d45",
".1 c #fb7049",
".V c #fb815e",
"#W c #fb8566",
"#e c #fb8868",
"#B c #fb8f70",
"#m c #fb9476",
"#t c #fb977b",
".L c #fc663b",
"#8 c #fc6940",
".K c #fc6b40",
"#6 c #fc6c44",
".J c #fc6e44",
".0 c #fc7953",
".W c #fc825f",
"#R c #fc8462",
"#V c #fc8666",
"#. c #fc9374",
"#H c #fc9d83",
"#7 c #fd6a41",
".Z c #fd7e5a",
".Y c #fd825e",
".X c #fd835f",
"#S c #fd8463",
"#U c #fd8464",
"#d c #fd9274",
"## c #fd9577",
"#C c #fd977a",
"#n c #fd9c81",
"#G c #fd9f85",
"#s c #fda289",
"#T c #fe8462",
"#a c #fe977a",
"#c c #fe987b",
"#b c #fe997c",
"#D c #fe997d",
"#E c #fe997e",
"#F c #fe9c82",
"#o c #fe9e84",
"#p c #fea187",
"#q c #fea48b",
"#r c #fea68d",
"Qt c None", //#ffffff",
"QtQtQt.#.a.b.c.d.e.f.gQtQtQtQt",
"QtQt.h.i.j.k.l.m.n.o.p.qQtQtQt",
"Qt.r.s.t.u.v.w.x.y.z.A.B.CQtQt",
".D.E.F.G.H.I.J.K.L.M.N.O.P.QQt",
".R.S.T.U.V.W.X.Y.Z.0.1.2.3.4.5",
".6.7.8.9#.###a#b#c#d#e#f#g#h#i",
"#j#k#l#m#n#o#p#q#r#s#t#u#v#w#x",
"#y#z#A#B#C#D#E#F#G#H#I#J#K#L#M",
"#N#O#P#Q#R#S#T#U#V#W#X#Y#Z#0#M",
"#1#2#3#4#5#6#7#8#9a.a#aaabac#x",
"adaeafagahaiajakalamanaoapaq#i",
"QtarasatauavawaxayazaAaBaCaDaE",
"QtQtaFaGaHaIaJaKaLaLaMaNaDaOQt",
"QtQtQtaPaQaRaSaTaUaVaWaDaOQtQt",
"QtQtQtQt.5aX#x#x#x#xaXaEQtQtQt"};


PScintilla::PScintilla(QWidget* parent, const char* name) : QsciScintilla(parent, name)
{
}

void PScintilla::dragEnterEvent(QDragEnterEvent* event)
{
	QStringList list;
	if(QUriDrag::decodeLocalFiles(event, list)) {
		int c = list.count();
		if(c==0) { return; }
		QString name = *list.at(0);
		//qDebug("name: "+name);
		if(name.isNull() || name=="") { return; }
		QFileInfo fi(name);
		if(fi.isSymLink()) { return; }
		if(fi.isDir()) { return; }
		if(fi.isFile()) { event->accept(); return; }
	}
}

void PScintilla::dropEvent(QDropEvent* event)
{
	QStringList list;
	if(QUriDrag::decodeLocalFiles(event, list)) {
		if(list.count()>0) {
			emit droppedFileName(*list.at(0));
		}
	}
}

PXmlValidator::PXmlValidator(QWidget* parent, const char* name) : QVBox(parent, name)
{
	setCaption("XML Schema Validator with Xerces");

	docFilePathName = "";

	currentPath = qApp->applicationDirPath();

	QString newPath=QDir::homeDirPath();
	#if defined (Q_OS_WIN32)
	if(newPath.length()<8) {
		newPath = "C:/Windows";
	}
	#endif
	if(newPath.length()>1 && (newPath.right(1)=="/" || newPath.right(1)=="\\")) {
		newPath = newPath.left(newPath.length()-1);
	}
	newPath += "/Desktop";
	newPath = QDir::convertSeparators(newPath);
	newPath.replace("\\","/");
	QFileInfo finf;
	finf.setFile(newPath);
	if(finf.exists() && finf.isDir()) {
		QDir dir(newPath);
		if(dir.isReadable()) {
			currentPath = newPath;
		}
	}

	editXML = new PScintilla(this, "editor");

	QObject::connect(editXML, SIGNAL(droppedFileName(QString)), this, SLOT(openValidate(QString)));

	printer = new QsciPrinter(QPrinter::HighResolution);

	QsciLexerXML *lex = new QsciLexerXML();

//	lex->setAutoIndentStyle(QsciScintilla::AiMaintain);
//	lex->setDefaultFont(QFont("Courier",8));

//	lex->setColor(QColor(255,0,0), QsciLexerXML::XMLTagEnd);
//	lex->setColor(QColor(255,0,0), QsciLexerXML::XMLEnd);
//	lex->setColor(QColor(255,0,0), QsciLexerXML::XMLStart);

//	lex->setColor(QColor(255,0,0), QsciLexerXML::HTMLDoubleQuotedString);
//	lex->setColor(QColor(255,0,0), QsciLexerXML::Attribute);
//	lex->setColor(QColor(255,0,0), QsciLexerXML::UnknownAttribute);
//	lex->setColor(QColor(255,0,0), QsciLexerXML::Tag);

	lex->setColor(QColor(255,0,0), -1);

	lex->refreshProperties();

	editXML->setLexer(lex);

	//editXML->autoCompleteFromAll();
	//editXML->setAutoCompletionFillupsEnabled(true);
	//editXML->setAutoCompletionSource(QsciScintilla::AcsAPIs);

	editXML->setUtf8(true);
	editXML->setWrapMode(QsciScintilla::WrapWord);

	editXML->setMarginWidth(0, "000");
	editXML->setMarginLineNumbers(0, true);

	editXML->setTabWidth(3);

	//editXML->setFolding(QsciScintilla::PlainFoldStyle);

	// Lesezeichen definieren
	editXML->markerDefine(QsciScintilla::SmallRectangle, 1); //RightTriangle
	editXML->setMarginSensitivity(0, true);
	editXML->setMarkerForegroundColor(QColor(220,220,220),-1);
	editXML->setMarkerBackgroundColor(QColor(250,0,0),-1);

	QHBox* hb = new QHBox(this);
	hb->setMargin(3);
	hb->setSpacing(3);
	hb->setFixedHeight(32);

	lblStatus = new QLabel("Zeile:, Zeichen: ", hb);

	lblStatus->setFrameStyle(QFrame::Raised | QFrame::Box);
	//lblStatus->setFrameStyle(QFrame::Raised | QFrame::StyledPanel);
	//lblStatus->setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);

	QObject::connect(editXML, SIGNAL(cursorPositionChanged(int,int)), this, SLOT(setStatus(int,int)));

	pbUndo = new QPushButton(hb);
	pbUndo->setFocusPolicy(QWidget::NoFocus);
	pbUndo->setPixmap(QPixmap(undoXPM));
	pbUndo->setFixedWidth(30);
	pbUndo->setFixedHeight(26);
	QToolTip::add(pbUndo, "Rckgngig");
	QObject::connect(pbUndo, SIGNAL(clicked()), this, SLOT(undo()));
	pbUndo->setEnabled(false);

	pbRedo = new QPushButton(hb);
	pbRedo->setFocusPolicy(QWidget::NoFocus);
	pbRedo->setPixmap(QPixmap(redoXPM));
	pbRedo->setFixedWidth(30);
	pbRedo->setFixedHeight(26);
	QToolTip::add(pbRedo, "Wiederherstellen");
	QObject::connect(pbRedo, SIGNAL(clicked()), this, SLOT(redo()));
	pbRedo->setEnabled(false);

	pbWrap = new QPushButton("R", hb);
	pbWrap->setFixedHeight(26);
	pbWrap->setFixedWidth(30);
	QObject::connect(pbWrap, SIGNAL(clicked()), this, SLOT(changeWrap()));

	pbDoc = new QPushButton("Dokumentation", hb);
	pbDoc->hide();

	pbLoad = new QPushButton(QPixmap(fileopen), "  ffnen", hb);
	pbLoad->setFixedHeight(26);
	QFontMetrics fmc(pbLoad->font());
	pbLoad->setFixedWidth(32 + fmc.width("  ffnen"));
	QObject::connect(pbLoad, SIGNAL(clicked()), this, SLOT(open()));

	pbSave = new QPushButton(QPixmap(filesave), "  Speichern", hb);
	pbSave->setFixedHeight(26);
	pbSave->setFixedWidth(32 + fmc.width("  Speichern"));
	QObject::connect(pbSave, SIGNAL(clicked()), this, SLOT(save()));

	pbSaveAs = new QPushButton(" Speichern unter ", hb);
	pbSaveAs->setFixedHeight(26);
	pbSaveAs->setFixedWidth(fmc.width("   Speichern unter  "));
	QObject::connect(pbSaveAs, SIGNAL(clicked()), this, SLOT(saveAs()));

	pbFormat = new QPushButton(" Formatieren ", hb);
	pbFormat->setFixedHeight(26);
	pbFormat->setFixedWidth(fmc.width("   Formatieren  "));
	QObject::connect(pbFormat, SIGNAL(clicked()), this, SLOT(format()));

	pbRevalid = new QPushButton("  Validieren  ", hb);
	pbRevalid->setFixedHeight(26);
	pbRevalid->setMinimumWidth(fmc.width("  Validieren  "));
	QObject::connect(pbRevalid, SIGNAL(clicked()), this, SLOT(revalidate()));

	pbPrint = new QPushButton(hb);
	pbPrint->setFocusPolicy(QWidget::NoFocus);
	pbPrint->setPixmap(QPixmap(fileprint));
	pbPrint->setFixedWidth(30);
	pbPrint->setFixedHeight(26);
	QToolTip::add(pbPrint, "Drucken");
	QObject::connect(pbPrint, SIGNAL(clicked()), this, SLOT(print()));

	QLabel* lbl = new QLabel(hb);
	lbl->setFixedHeight(26);

	editERR = new QTextEdit(this);

	editERR->setFixedHeight(100);

	QFont fo = editXML->font();
	fo.setPixelSize(14);
	editXML->setFont(fo);
	editERR->setFont(fo);

	QFontMetrics fm(lblStatus->font());
	int w = fm.width(" Zeile: 000000, Zeichen: 0000000 ");
	lblStatus->setFixedWidth(w);


	QObject::connect(editERR, SIGNAL(doubleClicked(int,int)), this, SLOT(showLine(int,int)));

	errorInvalid = "";
}

PXmlValidator::~PXmlValidator()
{
	delete printer;
	delete editXML;
	delete editERR;
	delete lblStatus;
	delete pbDoc;
	delete pbUndo;
	delete pbRedo;
	delete pbFormat;
	delete pbRevalid;
	delete pbPrint;
	delete pbSave;
	delete pbSaveAs;
	delete pbLoad;
	delete pbWrap;
}

void PXmlValidator::format()
{
	if(editXML->text().stripWhiteSpace()=="") { return; }

	if(!bFormatted) {

		QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

		QXmlSimpleReader reader;
		PXML_FormatParser* parser = new PXML_FormatParser();


		reader.setContentHandler(parser);
		reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
		reader.setFeature("http://xml.org/sax/features/namespaces", true);

		//QFile f; f.setName(fileNameXML);

		parser->reset();

		QXmlInputSource source;
		source.setData(editXML->text());
		reader.parse(source);

		QString prefix = parser->getDataStartPrefix();

		if(prefix!="") {
			prefix = "<"+prefix.stripWhiteSpace();
			int pos = editXML->text().find(prefix);
			prefix = editXML->text().left(pos).stripWhiteSpace();
			QString txR(QChar::QChar(10));
			prefix = prefix + txR;
		}

		QString data = prefix + parser->getFormattedString();

		editXML->setText(data);

		int cL = editXML->lines();
		QString nullen;
		nullen.fill('0', QString::number(cL).length()+1);
		editXML->setMarginWidth(0, nullen);
		deleteAllMarks();
		pbRedo->setEnabled(false);
		pbUndo->setEnabled(false);

		bFormatted = true;
		pbFormat->setToggleButton(true);
		//pbFormat->toggle();
		pbFormat->setOn(true);

		pbSave->setEnabled(false);

		revalidate();

		QApplication::restoreOverrideCursor();
	} else {
		bFormatted = false;

		pbFormat->setOn(false);
		pbFormat->setToggleButton(false);
		
		pbSave->setEnabled(true);
		openValidate(fileNameXML);
	}
}

void PXmlValidator::print()
{
	//printer->setFullPage(true); // druckt dann nicht mehr im Rahmen des A4-Blattes
	if(printer->setup(this)) { // printer dialog
//		statusBar()->message( "Printing..." );

		printer->printRange(editXML);

//		if(printer->printRange(e))
//			statusBar()->message( "Printing completed", 2000 );
//		else
//			statusBar()->message( "Error while printing", 2000 );
//		} else {
//		statusBar()->message( "Printing aborted", 2000 );
	}

/* formatPage() knnte berschrieben werden, um individueller zu drucken:

  virtual void QsciPrinter::formatPage(QPainter &painter, bool drawing, QRect &area, int pagenr)

  Format a page, by adding headers and footers for example, before the document text is drawn on it. 
  painter is the painter to be used to add customised text and graphics. 
  drawing is true if the page is actually being drawn rather than being sized. 
  painter drawing methods must only be called when drawing is true. area is the area of the page that 
  will be used to draw the text. This should be modified if it is necessary to reserve space for any 
  customised text or graphics. 
  By default the area is relative to the printable area of the page.
  Use QPrinter::setFullPage() because calling printRange() if you want to try and print over the whole page. 
  pagenr is the number of the page. The first page is numbered 1. 
*/
}

void PXmlValidator::updataReUndo()
{
	if(editXML->isUndoAvailable()) {
		pbUndo->setEnabled(true);
	} else {
		pbUndo->setEnabled(false);
	}
	if(editXML->isRedoAvailable()) {
		pbRedo->setEnabled(true);
	} else {
		pbRedo->setEnabled(false);
	}
}

void PXmlValidator::undo()
{
	editXML->undo();
}

void PXmlValidator::redo()
{
	editXML->redo();
}

void PXmlValidator::setCurrentPath(QString path)
{
	currentPath = path;
}

void PXmlValidator::setStatus(int line, int pos)
{
	lblStatus->setText("<b> Zeile: "+QString::number(line+1)+",<&nbsp;><&nbsp;>  Zeichen: "+QString::number(pos+1)+"</b>");
} 

void PXmlValidator::setDoc(QString filePathName)
{
	docFilePathName = filePathName;
	pbDoc->show();
	QObject::connect(pbDoc, SIGNAL(clicked()), this, SLOT(showDoc()));
}

void PXmlValidator::showDoc()
{
}

void PXmlValidator::changeWrap()
{
	if(editXML->wrapMode()==QsciScintilla::WrapWord) {

		editXML->setWrapMode(QsciScintilla::WrapNone);
	} else {
		editXML->setWrapMode(QsciScintilla::WrapWord);
	}
}

void PXmlValidator::revalidate()
{
	StdInParseERROR = "";
	errorInvalid = "";
//	deleteAllMarks();

	QFile f; f.setName(currentPath+"/temp_deleteMe.xml");

	if(f.open(IO_WriteOnly)) {
		QTextStream stream(&f);
		stream.setEncoding(QTextStream::UnicodeUTF8);
		stream << editXML->text();
		f.close();
	} else {
		return;
	}

	f.setName(fileNameXSD);
	if(fileNameXSD=="" || !f.exists()) {
		errorInvalid = "Schema Datei nicht gefunden";
		setErrorText(errorInvalid);
		QFile::remove(currentPath+"/temp_deleteMe.xml");
		return;
	}

	// Initialize the XML4C system
	try {
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		QString cerr = "Error during initialization! Message:\n" + QString(message);
		setErrorText(cerr);
		QFile::remove(currentPath+"/temp_deleteMe.xml");
		return;
	}

	//  Create a SAX parser object. Then, according to what we were told on
	//  the command line, set the options.
	SAXParser* parser = new SAXParser;
	parser->setValidationScheme(SAXParser::Val_Always);
	parser->setDoNamespaces(true);
	parser->setDoSchema(true);
	parser->setValidationSchemaFullChecking(true);

	//  Create our SAX handler object and install it on the parser, as the
	//  document and error handler. We are responsible for cleaning them
	//  up, but since its just stack based here, there's nothing special
	//  to do.
	StdInParseHandlers handler;
	parser->setDocumentHandler(&handler);
	parser->setErrorHandler(&handler);

	int errorCount = 0;
	//  Kick off the parse and catch any exceptions. Create a standard
	//  input input source and tell the parser to parse from that.
	try {
		QFile f; f.setName(currentPath+"/temp_deleteMe.xml");
		QFileInfo fi; fi.setFile(f);
		QString xmlFile = fi.absFilePath();
		parser->parse(xmlFile.ascii());
		errorCount = parser->getErrorCount();
	}
	catch(const OutOfMemoryException&) {
		QString cerr = "OutOfMemoryException";
		setErrorText(cerr);
		QFile::remove(currentPath+"/temp_deleteMe.xml");
		return;
	}
	catch(const XMLException& e) {
		char* message = XMLString::transcode(e.getMessage());
		QString cerr = "\nError during parsing: \n" + QString(message);
		setErrorText(cerr);
		QFile::remove(currentPath+"/temp_deleteMe.xml");
		return;
	}

	//  Delete the parser itself.  Must be done prior to calling Terminate, below.
	delete parser;

	QFile::remove(currentPath+"/temp_deleteMe.xml");

	XMLPlatformUtils::Terminate();

	if(errorCount>0) {
		// Fehler gefunden
		errorInvalid = StdInParseERROR;
		setErrorText(errorInvalid);
		return;
	} else {
		// keine Fehler gefunden
		setErrorText("");
	}
}

void PXmlValidator::save()
{
	if(bFormatted) { return; }
	if(editXML->text().stripWhiteSpace()=="") { return; }
	if(fileNameXML=="") { return; }

	QFile f; f.setName(fileNameXML);
	if(f.exists() && f.open(IO_WriteOnly)) {
		QTextStream stream(&f);
		stream.setEncoding(QTextStream::UnicodeUTF8);
		stream << editXML->text();
		f.close();
	}
}

void PXmlValidator::saveAs()
{
	if(editXML->text().stripWhiteSpace()=="") { return; }

	QString s = QFileDialog::getSaveFileName(currentPath, "XML-Dateien (*.xml)", this, "XML-Datei speichern", "Datei speichern unter");
	if(!s.isNull() && s!="") {
		QFile f; f.setName(s);
		if(f.exists()) {
			QMessageBox mBox("Speichern unter", "Die Datei existiert bereits. Soll diese berschrieben werden?", 
							 QMessageBox::Warning, QMessageBox::Yes, QMessageBox::Abort | QMessageBox::Default | QMessageBox::Escape, 0);
			mBox.setButtonText(QMessageBox::Yes, "Ja, speichern");
			mBox.setButtonText(QMessageBox::Abort, "Nein, abbrechen");
			int answer = mBox.exec();
			if(answer!=QMessageBox::Yes) {
				return;
			}
		}
		if(f.open(IO_WriteOnly)) {
			QTextStream stream(&f);
			stream.setEncoding(QTextStream::UnicodeUTF8);
			stream << editXML->text();
			f.close();
			QFileInfo fi; fi.setFile(f);
			fileNameXML = fi.absFilePath();
			setCaption("XML Schema Validator with Xerces: " + fileNameXML);
		}
	}
}

void PXmlValidator::open()
{
	QString s = QFileDialog::getOpenFileName(currentPath, "XML-Dateien (*.xml)", this, "XML-Datei ffnen", "Eine Datei whlen zum ffnen");

	setXML(s);
	//openValidate(s);
}

bool PXmlValidator::openValidate(QString filePathName)
{
	if(!filePathName.isNull() && filePathName!="") {

		QFile f; f.setName(filePathName);
		if(f.exists()) {

			QFileInfo fi; fi.setFile(f);
			setXML(fi.absFilePath());
			setErrorText(validate());
			return true;
		} else {
			return false;
		}
	}
	return false;
}

void PXmlValidator::showLine(int para, int pos)
{
	QString tx = editERR->text(para);
	tx = tx.mid(tx.find("Zeile: ")+7);

	QString txN = tx.left(tx.find(","));

	int line = txN.toInt()-1;
	editXML->setSelection(line, 1, line, 1);
	editXML->setCursorPosition(line, 0);
	if(line>-1) {
		if(tx.contains("value '") || tx.contains("element '")  || tx.contains("tag '")) {

			QString txF = "";
			if(tx.contains("value '")) {
				txF = tx.mid(tx.find("value '")+7);
			}
			if(tx.contains("element '")) {
				txF = tx.mid(tx.find("element '")+9);
			}
			if(tx.contains("tag '")) {
				txF = tx.mid(tx.find("tag '")+5);
			}

			txF = txF.left(txF.find("'"));
			int indexFrom = editXML->text(line).find(txF);
			editXML->findFirst(txF, false, true, false, false, true, line, -1, false); // findet und markiert den Text und setzt den Cursor dorthin
			editXML->setSelection(line, indexFrom, line, indexFrom+txF.length());
		}
		editXML->ensureLineVisible(line);
		editXML->ensureLineVisible(line-1);
		editXML->ensureCursorVisible();
	}
	editXML->setFocus();
}

bool PXmlValidator::isMark(int line)
{
	bool found= false; int i;
	for(i=0;i<(int)markList.count();i++) {
		if(*markList.at(i)==line) {
			found = true;
			break;
		}
	}
	return found;
}

void PXmlValidator::setMark(int line)
{
	bool found= false; int i;
	for(i=0;i<(int)markList.count();i++) {
		if(*markList.at(i)==line) {
			markList.remove(markList.at(i));
			editXML->markerDelete(line, 1);
			found = true;
			break;
		}
	}
	if(!found) {
		markList.append(line);
		qHeapSort(markList);
		editXML->markerAdd(line, 1);
	}
}

void PXmlValidator::setMark()
{
	int para, index;
	editXML->getCursorPosition(&para, &index);
	bool found= false; int i;
	for(i=0;i<(int)markList.count();i++) {
		if(*markList.at(i)==para) {
			markList.remove(markList.at(i));
			editXML->markerDelete(para, 1);
			found = true;
			break;
		}
	}
	if(!found) {
		markList.append(para);
		qHeapSort(markList);
		editXML->markerAdd(para, 1);
	}
}

void PXmlValidator::gotoMarkForward()
{
	int para, index;
	editXML->getCursorPosition(&para, &index);
	int i;
	if(markList.count()==1) {
		editXML->setCursorPosition(*markList.at(0),0);
		editXML->ensureLineVisible(*markList.at(0));
		editXML->ensureCursorVisible();
	} else {
		for(i=0;i<(int)markList.count();i++) {
			int row = *markList.at(i); 
			if(row>para) {
				editXML->setCursorPosition(row,0);
				editXML->ensureLineVisible(row);
				editXML->ensureCursorVisible();
				break;
			}
		}
	}
}

void PXmlValidator::gotoMarkBack()
{
	int para, index;
	editXML->getCursorPosition(&para, &index);
	int i;
	if(markList.count()==1) {
		editXML->setCursorPosition(*markList.at(0),0);
		editXML->ensureLineVisible(*markList.at(0));
		editXML->ensureCursorVisible();
	} else {
		for(i=(int)markList.count()-1;i>=0;i--) {
			int row = *markList.at(i); 
			if(row<para) {
				editXML->setCursorPosition(row,0);
				editXML->ensureLineVisible(row);
				editXML->ensureCursorVisible();
				break;
			}
		}
	}
}

void PXmlValidator::gotoMarkCont()
{
	int para, index;
	editXML->getCursorPosition(&para, &index);
	int i;
	if(markList.count()==1) {
		editXML->setCursorPosition(*markList.at(0),0);
		editXML->ensureLineVisible(*markList.at(0));
		editXML->ensureCursorVisible();
	} else {
		bool found = FALSE;
		for(i=0;i<(int)markList.count();i++) {
			int row = *markList.at(i); 
			if(row>para) {
				editXML->setCursorPosition(row,0);
				editXML->ensureLineVisible(row);
				editXML->ensureCursorVisible();
				found = TRUE;		
				break;
			}
		}
		if(markList.count()>0 && !found){
			int row = *markList.at(0);
			editXML->setCursorPosition(row,0);
			editXML->ensureLineVisible(row);
			editXML->ensureCursorVisible();
		}
	}
}

void PXmlValidator::deleteAllMarks()
{
	markList.clear();
	editXML->markerDeleteAll(1);
}

QString PXmlValidator::getXSDfromXML()
{
	//xsi:schemaLocation="http://www.xmlData.ch/xmlInvoice/XSD MDInvoiceResponse_400.xsd"
	if(editXML->text().lower().contains("schemalocation") && editXML->text().lower().contains(".xsd")) {
		int posE = editXML->text().find(".xsd")+4;
		int posA = editXML->text().findRev(" ", posE)+1;
		QString tx = editXML->text().mid(posA, posE-posA);
		return tx;
	}
	return "";
}

void PXmlValidator::setErrorText(QString errorText)
{
	editERR->setReadOnly(false);
	errorInvalid = "";
	
	QString txM = "<body bgcolor=#FFFFFF><br><b>Zum berprfen der Datei auf die Schaltflche 'Validieren' klicken.</b><br><br>";
	txM += "Die Validierung ist nur mglich, wenn eine XSD-Schema-Datei im selben Ordner liegt.</body>";
	editERR->setText(txM);
	deleteAllMarks();

	if(errorText.contains("Schema Datei nicht gefunden")) {

		QFile f; f.setName("error.png");
		QString imageStr = "";
		QString tableStr = "";
		if(f.exists()) {
			QFileInfo fi; fi.setFile(f);
			imageStr = "<table cellpadding=0 cellspacing=0 border=0><tr><td valign='center'><img src='" + fi.absFilePath() + "'></td><td valign='center'>";
			tableStr = "</td></tr></table>";
		}

		QString tx = "<html><body bgcolor=#DDBBBB>"+imageStr+"<b>XSD-Schema Datei <u>nicht</u> gefunden: </b>"+getXSDfromXML()+tableStr;
		tx += "<br>Die XML-Datei kann deshalb nicht geprft werden. ";
		tx += "Legen Sie die XSD-Schema-Datei in denselben Ordner wie die XML-Datei.</body></html>";

		if(getXSDfromXML()=="") {
			tx = "<html><body bgcolor=#DDBBBB>"+imageStr+"<b>Eintrag eines XSD-Schema <u>nicht</u> gefunden.</b>"+tableStr;
			tx += "<br>Die XML-Datei kann deshalb nicht geprft werden. ";
			tx += "Legen Sie eine XSD-Schema-Datei in denselben Ordner wie die XML-Datei und tragen Sie das XSD-Schema in die XML-Datei ein.</body></html>";
		}

		editERR->setText(tx);
		editERR->setFixedHeight(110);
		editERR->setReadOnly(true);
		return;
	}
	if(errorText!="") {

		QFile f; f.setName("error.png");
		QString imageStr = "<p>";
		QString tableStr = "</p>";
		if(f.exists()) {
			QFileInfo fi; fi.setFile(f);
			imageStr = "<table cellpadding=0 cellspacing=0 border=0><tr><td valign='center'><img src='" + fi.absFilePath() + "'></td><td valign='center'>";
			tableStr = "</td></tr></table>";
		}
		editERR->setText("<html><body bgcolor=#DDBBBB>"+imageStr+"<b> Die Datei ist gemss Schema <u>nicht</u> valide!</b>"+tableStr+errorText+"</body></html>");
		editERR->setFixedHeight(140);

		QStringList list = QStringList::split("Zeile: ", errorText, false);
		int i, iE = list.count();
		QString tx, txN;
		bool foundFirst = false;
		for(i=0;i<iE;i++) {
			tx = *list.at(i);
			txN = tx.left(tx.find(","));
			int line = txN.toInt()-1;
			if(line>-1) {
				if(!isMark(line)) {
					setMark(line);
				}
				if(!foundFirst) {
					if(tx.contains("value '") || tx.contains("element '") || tx.contains("tag '")) {
						QString txF = "";
						if(tx.contains("value '")) {
							txF = tx.mid(tx.find("value '")+7);
						}
						if(tx.contains("element '")) {
							txF = tx.mid(tx.find("element '")+9);
						}
						if(tx.contains("tag '")) {
							txF = tx.mid(tx.find("tag '")+5);
						}
						txF = txF.left(txF.find("'"));
						int indexFrom = editXML->text(line).find(txF);
						editXML->findFirst(txF, false, true, false, false, true, line, -1, false); // findet und markiert den Text und setzt den Cursor dorthin
						editXML->setSelection(line, indexFrom, line, indexFrom+txF.length());
					}
					editXML->ensureLineVisible(line);
					editXML->ensureLineVisible(line-1);
					foundFirst = true;
				}
			}
		}
	} else {
		editERR->setText("<body bgcolor=#BBDDBB><br><b>Die Datei ist gemss Schema valide!</b></body>");
		editERR->setFixedHeight(60);
	}
	editERR->setReadOnly(true);
}

void PXmlValidator::setXML(QString XMLfile)
{
	QFile f; f.setName(XMLfile);
	if(f.exists()) {
		if(f.open(IO_ReadOnly)) {

			bFormatted = false;
			pbFormat->setOn(false);
			pbFormat->setToggleButton(false);

			pbSave->setEnabled(true);

			QFileInfo fi; fi.setFile(f);

			fileNameXML = fi.absFilePath();

			QTextStream stream(&f);
			stream.setEncoding(QTextStream::UnicodeUTF8);
			QString tx = stream.read();
			
			editXML->setText(tx);

			currentPath = fi.dirPath(true);

			if(getXSDfromXML()=="") {
				fileNameXSD = "";
			} else {
				fileNameXSD = currentPath + "/" + getXSDfromXML();
			}
			
			QObject::connect(editXML, SIGNAL(textChanged()), this, SLOT(updataReUndo()));
			f.close();
			setCaption("XML Schema Validator with Xerces: " + fileNameXML);
			int cL = editXML->lines();
			QString nullen;
			nullen.fill('0', QString::number(cL).length()+1);
			editXML->setMarginWidth(0, nullen);
			deleteAllMarks();
			pbRedo->setEnabled(false);
			pbUndo->setEnabled(false);

			editERR->setText("");
		}
	}
}

QString PXmlValidator::getError()
{
	return errorInvalid;
}

QString PXmlValidator::validate()
{
	StdInParseERROR = "";
	errorInvalid = "";
	deleteAllMarks();

	QFile f; f.setName(fileNameXSD);
	if(fileNameXSD=="" || !f.exists()) {
		errorInvalid = "Schema Datei nicht gefunden"; 
		return errorInvalid;
	}

	// Initialize the XML4C system
	try {
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		QString cerr = "Error during initialization! Message:\n" + QString(message);
		return cerr;
	}

	//  Create a SAX parser object. Then, according to what we were told on
	//  the command line, set the options.
	SAXParser* parser = new SAXParser;
	parser->setValidationScheme(SAXParser::Val_Always);
	parser->setDoNamespaces(true);
	parser->setDoSchema(true);
	parser->setValidationSchemaFullChecking(true);

	//  Create our SAX handler object and install it on the parser, as the
	//  document and error handler. We are responsible for cleaning them
	//  up, but since its just stack based here, there's nothing special
	//  to do.
	StdInParseHandlers handler;
	parser->setDocumentHandler(&handler);
	parser->setErrorHandler(&handler);

	int errorCount = 0;
	//  Kick off the parse and catch any exceptions. Create a standard
	//  input input source and tell the parser to parse from that.
	try {
		QFile f; f.setName(fileNameXML);
		QFileInfo fi; fi.setFile(f);
		QString xmlFile = fi.absFilePath();
		parser->parse(xmlFile.ascii());
		errorCount = parser->getErrorCount();
	}
	catch(const OutOfMemoryException&) {
		QString cerr = "OutOfMemoryException";
		return cerr;
	}
	catch(const XMLException& e) {
		char* message = XMLString::transcode(e.getMessage());
		QString cerr = "\nError during parsing: \n" + QString(message);
		return cerr;
	}

	//  Delete the parser itself.  Must be done prior to calling Terminate, below.
	delete parser;

	XMLPlatformUtils::Terminate();

	if(errorCount>0) {
		// Fehler gefunden
		errorInvalid = StdInParseERROR;
		return StdInParseERROR;
	} else {
		// keine Fehler gefunden
		return "";
	}
}

QString format(QString fileXML) //static
{
	QFile f; f.setName(fileXML);
	if(!f.exists()) { return ""; }
	if(!f.open(IO_ReadOnly)) { return ""; }

	QXmlSimpleReader reader;
	PXML_FormatParser* parser = new PXML_FormatParser();

	reader.setContentHandler(parser);
	reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
	reader.setFeature("http://xml.org/sax/features/namespaces", true);

	parser->reset();

	QXmlInputSource source;
	
	QTextStream stream(&f);
	stream.setEncoding(QTextStream::UnicodeUTF8);
	QString txData = stream.read();
	f.close();

	source.setData(txData);
	reader.parse(source);

	QString prefix = parser->getDataStartPrefix();

	if(prefix!="") {
		prefix = "<"+prefix.stripWhiteSpace();
		int pos = txData.find(prefix);
		prefix = txData.left(pos).stripWhiteSpace();
		QString txR(QChar::QChar(10));
		prefix = prefix + txR;
	}

	txData = prefix + parser->getFormattedString();
	return txData;
}

QString PXmlValidator::validate(QString XMLfile) // static
{
	StdInParseERROR = "";

	// Initialize the XML4C system
	try {
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& toCatch) {
		char* message = XMLString::transcode(toCatch.getMessage());
		QString cerr = "Error during initialization! Message:\n" + QString(message);
		return cerr;
	}

	//  Create a SAX parser object. Then, according to what we were told on
	//  the command line, set the options.
	SAXParser* parser = new SAXParser;
	parser->setValidationScheme(SAXParser::Val_Always);
	parser->setDoNamespaces(true);
	parser->setDoSchema(true);
	parser->setValidationSchemaFullChecking(true);

	//  Create our SAX handler object and install it on the parser, as the
	//  document and error handler. We are responsible for cleaning them
	//  up, but since its just stack based here, there's nothing special
	//  to do.
	StdInParseHandlers handler;
	parser->setDocumentHandler(&handler);
	parser->setErrorHandler(&handler);

	int errorCount = 0;
	//  Kick off the parse and catch any exceptions. Create a standard
	//  input input source and tell the parser to parse from that.
	try {
		QFile f; f.setName(XMLfile);
		QFileInfo fi; fi.setFile(f);
		QString xmlPathName = fi.absFilePath();
		parser->parse(xmlPathName.ascii());
		errorCount = parser->getErrorCount();
	}
	catch(const OutOfMemoryException&) {
		QString cerr = "OutOfMemoryException";
		return cerr;
	}
	catch(const XMLException& e) {
		char* message = XMLString::transcode(e.getMessage());
		QString cerr = "\nError during parsing: \n" + QString(message);
		return cerr;
	}

	//  Delete the parser itself.  Must be done prior to calling Terminate, below.
	delete parser;

	XMLPlatformUtils::Terminate();

	if(errorCount>0) {
		// Fehler gefunden
		return StdInParseERROR;
	} else {
		// keine Fehler gefunden
		return "";
	}
}
